#!/usr/bin/env python

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Authors: Isaiah Heyer (freshapplepy@gmail.com)

# Copyright (C) 2008 Isaiah Heyer

import pygtk
pygtk.require('2.0')
import gtk
from xml.etree import ElementTree as ET
import commands
import os
import shutil

import copy
import indent
import utils

class Theme(object):
    
    def __init__(self,filename,restore=False):

        self.filename = filename

        dir = self.get_dir()

        if restore:
            shutil.copy(utils.join(dir,".metacity-theme-1.xml"),filename)


        self.tree = ET.parse(filename)
        self.tree.write(self.filename)
        
        self.saved_file = utils.join(dir,".sav_metacity-theme-1.xml~")
        
        if not restore:
            self.tree.write(self.saved_file)

        ### create a backup file

        self.backup_file = utils.join(self.get_dir(),
                ".metacity-theme-1.xml")

        self.tree.write(self.backup_file)
        
        self.real_focused_style = None
        self.real_unfocused_style = None
        
        self.geometry_d = ['left_width',
                'right_width',
                'bottom_height',
                'left_titlebar_edge',
                'right_titlebar_edge',
                'button_width',
                'button_height',
                'title_vertical_pad']

        self.geometry_borders = ['title_border','button_border']

        self.button_order = ['close',
                'maximize',
                'minimize',
                'menu',
                'left_left_background',
                'left_middle_background',
                'left_right_background',
                'right_left_background',
                'right_middle_background',
                'right_right_background']
        
        self.piece_order = ['entire_background',
                'titlebar',
                'titlebar_middle',
                'left_titlebar_edge',
                'right_titlebar_edge',
                'top_titlebar_edge',
                'bottom_titlebar_edge',
                'title',
                'left_edge',
                'right_edge',
                'bottom_edge',
                'overlay']

        self.load()

    def get_dir(self):
        """
        Returns the directory of the self.filename
        """
        return os.path.dirname(self.filename)

    def set_theme_file(self,filename):
        """
        Changes filename for theme in case
        of theme renaming
        """

        self.filename = filename
        self.backup_file = utils.join(self.get_dir(),
                ".metacity-theme-1.xml")

        self.saved_file = utils.join(self.get_dir(),
                ".sav_metacity-theme-1.xml~")

    def load(self):
        

        self.root = self.tree.getroot()
        self.elements = self.root.getiterator('metacity_theme')
        self.elements = self.elements[0]
        
        self.info_elem = None
    
        self.real_button_drawops = {}

        self.constant_list = []
        self.draw_ops_list = []
        self.frame_geometry_list = []
        self.frame_style_list = []
        self.frame_style_set_list = []
        self.window_list = []

        self.constant = {}
        self.draw_ops = {}
        self.frame_geometry = {}
        self.fg_children = {}

        self.frame_style = {}
        self.fs_children = {}

        

        self.frame_style_set = {}
        self.fss_children = {}

        self.window = {}
        self.Element = {}
        
        
        for elem in self.elements:
            e = elem.attrib
            tag = elem.tag
             
            if tag == "info":
                self.info_elem = elem

            elif tag == "constant":
                self.constant[e['name']] = elem
                self.constant_list.append(elem)

            elif tag == "draw_ops":
                self.draw_ops[e['name']] = elem
                self.draw_ops_list.append(elem)
        
            elif tag == "frame_geometry":
                self.frame_geometry[e['name']] = elem
                self.frame_geometry_list.append(elem)
                
                do_not_add = []

                fg_values = []
                for child in elem:
                    if child.attrib['name'] == 'button':
                        
                        do_not_add.append('button_width')
                        do_not_add.append('button_height')

                    elif child.attrib['name'] in ['button_width',
                            'button_height']:
                        
                        do_not_add.append('button')

                    fg_values.append(child.attrib['name'])


                if elem.attrib.has_key('parent'):
                    parent_name = elem.attrib['parent']
                    parent = self.frame_geometry[parent_name]
                    cl = []
                    for child in parent:
                        name = child.attrib['name']
                        if not name in fg_values and not name in do_not_add:
                            elem.append(child)
                    
                    elem.attrib.pop('parent') ## remove inheritance

                self.fg_children[e['name']] = {}
                for child in elem:
                    self.fg_children[e['name']][child.attrib['name']] = child



            elif tag == "frame_style":
                
                self.frame_style[e['name']] = elem
                self.frame_style_list.append(elem)

                children = {}

                child_list = [child for child in elem]
                for child in child_list:
                    attr = child.attrib.copy()
                    if child.tag == "button":
                        key = attr['function']+attr['state']
                    else:
                        key = attr['position']

                    elem.remove(child)
                    children[key] = child
                
                if elem.attrib.has_key('parent'):
                    parent_name = elem.attrib['parent']
                    parent = self.frame_style[parent_name]

                    parent_child_list = [child for child in parent]
                    cl = []
                    for c in parent_child_list:
                        attr = c.attrib.copy()
                        if c.tag == "button":
                            key = attr['function']+attr['state']
                        else:
                            key = attr['position']

                        if not key in children.keys():
                            children[key] = c

                ## check for optional attributes and add them if not there
                for b in self.button_order:
                    for state in ['normal','prelight','pressed']:
                        key = b+state
                        if not key in children.keys():
                            new_elem = ET.Element("button",
                                {'function':b,'state':state,'draw_ops':'~FOO_do~'})
                            elem.append(new_elem)
                        else:
                            elem.append(children[key])

                for p in self.piece_order:
                    if not p in children.keys():
                        new_elem = ET.Element("piece",
                                {'position':p,'draw_ops':'~FOO_do~'})
                        elem.append(new_elem)

                    else:
                        elem.append(children[p])
                    
                
                ### now add all children to self.frame_style_children
                self.fs_children[e['name']] = {}
                
                for b in self.button_order:
                    self.fs_children[e['name']][b] = {}
        
                for p in self.piece_order:
                    self.fs_children[e['name']][p] = {}

                for child in elem:
                    a = child.attrib
                    if child.tag == "button":
                        self.fs_children[e['name']] [a['function']] [a['state']] = child
                    else:
                        self.fs_children[e['name']] [a['position']] = child
                

            elif tag == "frame_style_set":
                self.frame_style_set[e['name']] = elem
                self.frame_style_set_list.append(elem)
                
                self.fss_children[e['name']] = {'yes':{},'no':{}}

                for child in elem:
                    a = child.attrib
                    self.fss_children[e['name']] [a['focus']] [a['state']] = child

            elif tag == "window":
                self.window[e['type']] = elem
                self.window_list.append(elem)
        
        fss = self.window['normal'].attrib['style_set']

        self.normal_frame_style = self.fss_children[fss]['yes']['normal'].attrib['style']

        #### add ~FOO~ elements
        if not "~FOO_do~" in self.draw_ops.keys():
            self.make_drawops('~FOO_do~',True)

        if not "~FOO_fg~" in self.frame_geometry.keys():
            self.make_geometry('~FOO_fg~')

        if not "~FOO_fs~" in self.frame_style.keys():
            self.make_frame_style('~FOO_fs~')

        #if not "~FOO_fss~" in self.frame_style_set.keys():
        #    self.make_frame_style_set('~FOO_fss~')
    
        # reorder the elements
        self.reorder()
        
    def does_include(self,elem):
        for child in elem:
            if child.tag == "include":
                return child.attrib['name']
            else:
                continue
        return []

    
    def is_valid_ref(self,elem,parent_elem):
        """
        Return true if elem.tag == "include" and its attrib 'name'
        reference is valid
        """
        ref = elem.attrib['name']
        try:
            ref_elem = self.draw_ops[ref]
            ref_i = self.draw_ops_list.index(ref_elem)
            pelem_i = self.draw_ops_list.index(parent_elem)
        
        except Exception,data:
            print data
            return False

        else:
            if ref_i < pelem_i:
                return True
            else:
                return False


    def reorder(self):
        order = []

        for elem in self.constant_list:
            order.append(elem)
        
        
        for elem in self.draw_ops_list:
            order.append(elem)

        for elem in self.frame_geometry_list:
            if not elem.attrib.has_key('parent'):
                order.append(elem)
        for elem in self.frame_geometry_list:
            if elem.attrib.has_key('parent'):
                order.append(elem)

        for elem in self.frame_style_list:
            if not elem.attrib.has_key('parent'):
                order.append(elem)
        for elem in self.frame_style_list:
            if elem.attrib.has_key('parent'):
                order.append(elem)

        for elem in self.frame_style_set_list:
            if not elem.attrib.has_key('parent'):
                order.append(elem)
        for elem in self.frame_style_set_list:
            if elem.attrib.has_key('parent'):
                order.append(elem)

        for elem in self.window_list:
            order.append(elem)

        for elem in order:
            self.root.remove(elem)
            self.root.append(elem)
        

    def set_preview_state(self,win_type,focus,state):
        
        # reset the preview button states
        self.reset_preview_button_states()

        fss = self.window['normal'].attrib['style_set']

        normal_focused_frame = self.fss_children[fss]['yes']['normal']
        
        ## reset the frame style
        normal_focused_frame.attrib['style'] = self.normal_frame_style
        
        frame_elem = self.Get_Element(win_type,focus,state)
        
        normal_focused_frame.attrib['style'] = frame_elem.attrib['style']
        
        
    def set_preview_button_state(self,win_type,focus,state,function,Bstate):
        
        self.reset_preview_button_states()

        Welem = self.window[win_type]
        fss_name = Welem.attrib['style_set']
        fs_name = self.fss_children[fss_name][focus][state].attrib['style']
        
        button_elem = self.fs_children[fs_name][function][Bstate]
        preview_button_elem = self.fs_children[fs_name][function]['normal']
        
        if Bstate == "normal":
            try:
                preview_button_elem.attrib =\
                    self.real_button_drawops[fs_name][function]

            except KeyError,data:
                pass
        else:
            if not self.real_button_drawops.has_key(fs_name):
                self.real_button_drawops[fs_name] = {}

            self.real_button_drawops[fs_name][function] =\
                preview_button_elem.attrib.copy()
        
        preview_button_elem.attrib['draw_ops'] = button_elem.attrib['draw_ops']

    def reset_preview_button_states(self):

        ##set button elements drawops attribute to what it "really" is
        for fs_name,d in self.real_button_drawops.items():
            for function,attrib in d.items():
                button_elem = self.fs_children[fs_name][function]['normal']
                button_elem.attrib = attrib

    
    def set_button_drawops(self,win_type,focus,state,function,Bstate,
            drawops_name):
       
        Welem = self.window[win_type]
        fss_name = Welem.attrib['style_set']
        fs_name = self.fss_children[fss_name][focus][state].attrib['style']
        
        button_elem = self.fs_children[fs_name][function][Bstate]
        
        button_elem.attrib['draw_ops'] = drawops_name

        if Bstate == "normal":
            if not self.real_button_drawops.has_key(fs_name):
                self.real_button_drawops[fs_name] = {}

            
            self.real_button_drawops[fs_name][function] =\
                    button_elem.attrib.copy()

    def get_fs_list(self):
        return [fs.attrib['name'] for fs in self.frame_style_list]
    
    def get_fss_list(self):
        return [fss.attrib['name'] for fss in self.frame_style_set_list]

    def set_frame_style(self,win_type,elem,fs_name):

        fss = self.window['normal'].attrib['style_set']
        normal_focused_frame = self.fss_children[fss]['yes']['normal']
        normal_focused_frame.attrib['style'] = self.normal_frame_style
        
        ### actually set the frame style
        old_fs_name = elem.attrib['style']
        elem.attrib['style'] = fs_name
        ####

        preview_frame_style = normal_focused_frame.attrib['style']
        self.normal_frame_style = normal_focused_frame.attrib['style']

        if preview_frame_style == old_fs_name:
            normal_focused_frame.attrib['style'] = fs_name
        
    def set_frame_style_set(self,elem,fss_name):
        
        win_type = elem.attrib['type']


        fss = self.window['normal'].attrib['style_set']
        normal_focused_frame = self.fss_children[fss]['yes']['normal']
        
        if win_type == "normal":
            normal_focused_frame.attrib['style'] = self.normal_frame_style

            normal_focused_frame = self.fss_children[fss_name]['yes']['normal']
            self.normal_frame_style = normal_focused_frame.attrib['style']
        
        elem.attrib['style_set'] = fss_name


    def reorder_drawops(self,order_list,break_include=False):
        
        deps = []
        for name in order_list:
            elem = self.draw_ops[name]
            includes = self.does_include(elem)
            
            if includes:
                if order_list.index(includes) > order_list.index(name):
                    if break_include:
                        for child in elem:
                            if child.tag == "include":
                                child.attrib['name'] = "~FOO_do~"
                    else:
                        deps.append((name,includes))
        if deps:
            return deps

        pos = len(self.constant_list) + 1
        
        while self.draw_ops_list:
            self.draw_ops_list.pop()

        for name in order_list:
            elem = self.draw_ops[name]
            self.draw_ops_list.append(elem)

            self.root.remove(elem)
            self.root.insert(pos,elem)
            pos += 1
        
        return deps

    def make_drawops(self,name,top=False):
        if name in self.draw_ops.keys():
            print "name reuse error"
            return False

        new_elem = ET.Element("draw_ops",{'name':name})
        
        i = 0
        tag = self.root[i].tag
        while tag != "draw_ops":
            i += 1
            tag = self.root[i].tag
        
        if top:
            pos = i
        
        else:
            pos = i + len(self.draw_ops_list)

        self.root.insert(pos,new_elem)

        self.draw_ops[name] = new_elem
        if top:
            self.draw_ops_list.insert(0,new_elem)
        else:
            self.draw_ops_list.append(new_elem)


    def make_geometry(self,name):
        new_elem = ET.Element("frame_geometry",{'name':name})

        for g in self.geometry_d:
            new_child = ET.Element("distance",{'name':g,'value':'10'})

            new_elem.append(new_child)

        for b in self.geometry_borders:
            new_child = ET.Element("border",{'name':b,'left':'4',
                'right':'4','top':'4','bottom':'4'})
            
            new_elem.append(new_child)
        
        i = 0
        tag = self.root[i].tag
        while tag != "frame_geometry":
            i += 1
            tag = self.root[i].tag

        pos = i + len(self.frame_geometry_list)
        self.root.insert(pos,new_elem)

        self.frame_geometry[name] = new_elem
        self.frame_geometry_list.append(new_elem)
            
    def make_frame_style(self,name):
        new_elem = ET.Element("frame_style",
            {"name":name,"geometry":'~FOO_fg~'})
                    
        for b in self.button_order:
            for state in ['normal','prelight','pressed']:
                new_child = ET.Element("button",
                    {'function':b,'state':state,'draw_ops':'~FOO_do~'})
                new_elem.append(new_child)

        for p in self.piece_order:
            new_child = ET.Element("piece",
                    {'position':p,'draw_ops':'~FOO_do~'})
                        
            new_elem.append(new_child)

        i = 0
        tag = self.root[i].tag
        while tag != "frame_style":
            i += 1
            tag = self.root[i].tag

        pos = i + len(self.frame_style_list)
        self.root.insert(pos,new_elem)

        self.frame_style[name] = new_elem
        self.frame_style_list.append(new_elem)
        
        ### now add all children to self.frame_style_children
        self.fs_children[name] = {}
                
        for b in self.button_order:
            self.fs_children[name][b] = {}
        
        for p in self.piece_order:
            self.fs_children[name][p] = {}

        for child in new_elem:
            a = child.attrib
            if child.tag == "button":
                self.fs_children[name] [a['function']] [a['state']] = child
            else:
                self.fs_children[name] [a['position']] = child

    def make_frame_style_set(self,name):
        new_elem = ET.Element("frame_style_set",{'name':name})
            
        for focus in ['yes','no']:

            new_child = ET.Element('frame',{'focus':focus,'resize':'both',
                'state':'normal','style':'~FOO_fs~'})
                
            new_elem.append(new_child)

            new_child = ET.Element('frame',{'focus':focus,
                    'state':'maximized','style':'~FOO_fs~'})

            new_elem.append(new_child)

            new_child = ET.Element('frame',{'focus':focus,
                    'state':'shaded','style':'~FOO_fs~'})

            new_elem.append(new_child)

            new_child = ET.Element('frame',{'focus':focus,
                    'state':'maximized_and_shaded','style':'~FOO_fs~'})
                
            new_elem.append(new_child)

        i = 0
        tag = self.root[i].tag
        while tag != "frame_style_set":
            i += 1
            tag = self.root[i].tag

        pos = i + len(self.frame_style_set_list)
        self.root.insert(pos,new_elem)

        self.frame_style_set[name] = new_elem
        self.frame_style_set_list.append(new_elem)
        
        self.fss_children[name] = {'yes':{},'no':{}}

        for child in new_elem:
            a = child.attrib
            self.fss_children[name] [a['focus']] [a['state']] = child

    
    def make_constant(self,name,value):
        if name in self.constant.keys():
            print "name reuse error"
            return False

        
        new_elem = ET.Element("constant",{'name':name,'value':value})
        
        self.root.insert(1,new_elem)

        self.constant[name] = new_elem
        self.constant_list.append(new_elem)

    
    def get_all_elements(self):
        return [child for child in self.root]
    
    def _find_constant_references(self,name):
        refs = []

        for do in self.draw_ops_list:
            for child in do:
                for value in child.attrib.values():
                    if name in value:
                        refs.append((do.tag,do.attrib['name']))

        return refs

    def _find_references(self,elem_list,attrib,string):
        refs = []
        for elem in elem_list:
            if elem.attrib.has_key(attrib):
                if elem.attrib[attrib] == string:
                    if not attrib == "name":
                        if elem.tag != "window":
                            refs.append((elem.tag,elem.attrib['name']))
                        else:
                            refs.append((elem.tag,elem.attrib['type']))


            for child in elem:
                if child.attrib.has_key(attrib):
                    if child.attrib[attrib] == string:
                        refs.append((elem.tag,elem.attrib['name']))

        return refs

    def remove_element(self,name,type):

        if type == "draw_ops":
            elem_dict = self.draw_ops
            elem_list1 = self.draw_ops_list
            str1 = "name"
            elem_list2 = self.frame_style_list
            str2 = "draw_ops"
        
        elif type == "frame_geometry":
            elem_dict = self.frame_geometry
            elem_list1 = self.frame_geometry_list
            str1 = ''
            elem_list2 = self.frame_style_list
            str2 = "geometry"
        
        elif type == "frame_style":
            elem_dict = self.frame_style
            elem_list1 = self.frame_style_list
            str1 = 'parent'
            elem_list2 = self.frame_style_set_list
            str2 = "style"
        
        elif type == "frame_style_set":
            elem_dict = self.frame_style_set
            elem_list1 = self.frame_style_set_list
            str1 = ''
            elem_list2 = self.window_list
            str2 = "style_set"
        
        elif type == "constant":
            refs = self._find_constant_references(name)
            if refs:
                return refs

            element = self.constant[name]
            self.root.remove(element)
            self.constant_list.pop(self.constant_list.index(element))
            self.constant.pop(name)
            
            return refs

            
        element = elem_dict[name]
        
        refs1 = self._find_references(elem_list1,str1,name)
        refs2 = self._find_references(elem_list2,str2,name)
        
        refs = refs1 + refs2

        if refs:
            return refs

        self.root.remove(element)
        elem_list1.pop(elem_list1.index(element))
        elem_dict.pop(name)
        
        return refs

    def get_drawops_list(self):
        return [elem.attrib['name'] for elem in self.draw_ops_list]
    
    def get_geometry_list(self):
        return [elem.attrib['name'] for elem in self.frame_geometry_list]

    def get_frame_style_list(self):
        return [elem.attrib['name'] for elem in self.frame_style_list]

    def get_frame_style_set_list(self):
        return [elem.attrib['name'] for elem in self.frame_style_set_list]

    def get_constant_list(self):
        return [con.attrib['name'] for con in self.constant_list]
    
    def get_info_element(self):
        return self.info_elem


    def Get_Element(self,win_type,focus=None,state=None,geometry=None,
                            function=None,Bstate=None):

        Welem = self.window[win_type]
        fss = Welem.attrib['style_set']
        
        if focus and state:

            frame_elem = self.fss_children[fss][focus][state]
            fs_name = frame_elem.attrib['style']
            fs = self.frame_style[fs_name]
            
            if geometry:
                return self.frame_geometry[fs.attrib['geometry']]

            elif function:
                if Bstate:
                    button_elem = self.fs_children[fs_name][function][Bstate]
                    return button_elem
                else:
                    piece_elem = self.fs_children[fs_name][function]
                    return piece_elem
            else:
                return frame_elem

        else:
            return self.window[win_type]
    
    def get_includable(self,name):
        l = []
        for i in self.elements:
            if i.tag != "draw_ops":
                continue

            if i.attrib['name'] != name:
                l.append(i.attrib['name'])
            else:
                l.sort()
                return l
    
    def save(self):
        """
        call when user saves the theme
        """

        fss = self.window['normal'].attrib['style_set']

        normal_focused_frame = self.fss_children[fss]['yes']['normal']
        old_style = normal_focused_frame.attrib['style']

        normal_focused_frame.attrib['style'] = self.normal_frame_style
        
        old_values = []
        ##set button elements drawops attribute to what it "really" is
        for fs_name,d in self.real_button_drawops.items():
            for function,attrib in d.items():
                button_elem = self.fs_children[fs_name][function]['normal']

                old_values.append((button_elem,button_elem.attrib.copy()))

                button_elem.attrib = attrib

        ##################
        self.write()
        self.tree.write(self.saved_file)
        ##################

        for elem,attrib in old_values:
            elem.attrib = attrib


        normal_focused_frame.attrib['style'] = old_style

    def write_backup(self):
        
        fss = self.window['normal'].attrib['style_set']

        normal_focused_frame = self.fss_children[fss]['yes']['normal']
        old_style = normal_focused_frame.attrib['style']

        normal_focused_frame.attrib['style'] = self.normal_frame_style
        
        old_values = []
        ##set button elements drawops attribute to what it "really" is
        for fs_name,d in self.real_button_drawops.items():
            for function,attrib in d.items():
                button_elem = self.fs_children[fs_name][function]['normal']

                old_values.append((button_elem,button_elem.attrib.copy()))

                button_elem.attrib = attrib


        ##################
        self.tree.write(self.backup_file)
        ##################
        
        for elem,attrib in old_values:
            elem.attrib = attrib

        normal_focused_frame.attrib['style'] = old_style
        

    def write(self):
        write_tree = copy.deepcopy(self.tree)

        #self.reorder()
        self.tree.write(self.filename)

    
    def get_needs_save(self):

        return bool(commands.getoutput('diff '+
            '"'+self.backup_file+'" "'+self.saved_file+'"'))

    def quit(self):
        """
        This function should be called when the program is closed
        to restore the theme xml file to its last saved state
        """
        
        shutil.copy(self.saved_file,self.filename)
        indent.fix(self.filename)
                        
