#!/usr/bin/env python

from os.path import abspath, dirname, join, pardir, sep


# here we define the path constants so that other modules can use it.
# this allows us to get access to the shared files without having to
# know the actual location, we just use the location of the current
# file and use paths relative to that.

## check to see if this is running from a local directory or 
## from an installation
parent_dir = abspath(join(dirname(__file__),pardir))

if parent_dir.split(sep)[-1] == "site-packages":
    prefix = join(dirname(__file__),pardir,pardir,pardir,pardir)

else:
    prefix = parent_dir

SHARED_FILES = abspath(join(prefix,'share'))

GLADE_PATH = join(SHARED_FILES,'metacity-themer','glade')
LOCALE_PATH = join(SHARED_FILES,'locale')
RESOURCE_PATH = join(SHARED_FILES,'metacity-themer','res')
ICON_PATH = join(SHARED_FILES,'icons')

THEME_TEST_PATH = abspath(dirname(__file__))


GETTEXT_DOMAIN = 'metacity-themer'

# setup PyGTK by requiring GTK2
import pygtk
pygtk.require('2.0')

# set up the gettext system and locales
import gtk
import gtk.glade
import gettext
import locale
import os
import shutil
import datetime
import time

## Configuration file path
CONF_DIR = os.path.join(os.getenv("HOME"),".metacity-themer")
if not os.path.isdir(CONF_DIR):
    os.mkdir(CONF_DIR)

## Crashed themes file
CT_FILENAME = os.path.join(CONF_DIR,'ct')
## Log file stuff

today = datetime.date.today().day

LOG_DIR = os.path.join(CONF_DIR,"LOGS")

if not os.path.isdir(LOG_DIR):
    os.mkdir(LOG_DIR)

DAY_LOG_DIR = os.path.join(CONF_DIR,LOG_DIR,str(today))

if not os.path.isdir(DAY_LOG_DIR):
    os.mkdir(DAY_LOG_DIR)

dl = os.listdir(LOG_DIR)
for d in dl:
    if d.isdigit() and int(d) != today:
        log_dir = os.path.join(LOG_DIR,d)
        if os.path.isdir(log_dir):
            shutil.rmtree(log_dir)

LOG_FILENAME = os.path.join(CONF_DIR,DAY_LOG_DIR,
        "LOG_"+str(int(time.time())))

###########################

### Element database filename

DB_FILENAME = os.path.join(CONF_DIR,"elem_db")

## delete the database if there aren't any instances open
import commands

## Process name
PID = 'metacity-themer'

num_of_proc = 0 ## Number of running instances of this program

for p in commands.getoutput('ps -A').split('\n'):
    if PID in p:
        num_of_proc += 1

if num_of_proc < 2:
    if os.path.isfile(DB_FILENAME):
        os.remove(DB_FILENAME)

locale.setlocale(locale.LC_ALL,'')

for module in gtk.glade, gettext:
    module.bindtextdomain(GETTEXT_DOMAIN, LOCALE_PATH)
    module.textdomain(GETTEXT_DOMAIN)

# register the gettext function for the whole interpreter as "_"
import __builtin__
__builtin__._ = gettext.gettext

# import the main function
from metacity_themer.main import main


