# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Authors: Isaiah Heyer (freshapplepy@gmail.com)

# Copyright (C) 2008 Isaiah Heyer


import gtk
import meta_expr
import commands
import os

import attribs
import dialogs
import undo
import utils

from xml.etree import ElementTree as ET


opt_args = {'alpha':'1.0','colorize':"#000000",'dash_on_length':'0',
            'dash_off_length':'0','width':'0','height':'0',
            'filled':'false','x':'0','y':'0',
            'tile_xoffset':'0','tile_yoffset':'0'}


class AttribBox(gtk.HBox):

    def __init__(self,refresh_func,element,attrib,drop_list=[],opt=False):
        
        gtk.HBox.__init__(self)

        self.element = element
        self.attrib = attrib

        
        self.refresh = refresh_func
        self.drop_list = drop_list
        self.opt = opt
        
        self.user_set = True

        if element.attrib.has_key(attrib):
            self.has_attr = True
        else:
            self.has_attr = False

        if opt:
            check = gtk.CheckButton(attrib+":")
            self.check = check
        
            label = check.get_child()
            self.label = label
        
            label_text = attrib.split("_")
            label_text = " ".join(label_text)
            label.set_text(label_text+":")

            label.set_alignment(0.90,0.50)
            if len(label_text) > 10:
                label.set_size_request(86,-1)
            else:
                label.set_size_request(76,-1)

            if self.has_attr:
                check.set_active(True)
            
            
                        
            check.connect("toggled",self.check_callback)
        
            self.pack_start(check,False,False)
            check.show()

        else:
            label_text = attrib.split("_")
            label_text = " ".join(label_text)
            label = gtk.Label(label_text+":")
            self.label = label

            label.set_alignment(0.90,0.50)
            
            if len(label_text) > 10:
                label.set_size_request(100+len(label_text)-10,-1)
            else:
                label.set_size_request(100,-1)


            label.set_size_request(100,-1)
            self.pack_start(label,False,False)

        
        self.label.show()

        cEntry = gtk.combo_box_entry_new_text()
        Entry = cEntry.get_children()[0]
        
        self.cEntry = cEntry
        self.Entry = Entry

        if self.has_attr:
            self.Entry.set_text(self.element.attrib[self.attrib])
            
        else:
            self.label.set_sensitive(False)
            self.cEntry.set_sensitive(False)

            v = opt_args[self.attrib]
            self.Entry.set_text(v)

        for text in drop_list:
            cEntry.append_text(text)

        self.entry_old_value = Entry.get_text()

        
        cEntry.connect("changed",self.on_cEntry_changed)

        Entry.connect("focus-in-event",self.on_focus_in)
        Entry.connect("focus-out-event",self.on_focus_out)
        Entry.connect("activate",self.on_activate)

        self.pack_start(cEntry,True,True)

        self.cEntry.show()

        if not self.has_attr:
            label.set_sensitive(False)
            Entry.set_sensitive(False)

        
    def on_cEntry_changed(self,widget,data=None):
        if widget.get_active() > -1:
            entry = widget.get_child()
            entry.activate()
    
        
    def on_focus_in(self,widget,event):
        self.entry_old_value = widget.get_text()

    def on_focus_out(self,widget,event):
        widget.activate()

    def on_activate(self,widget,data=None):
        text = widget.get_text()
        text = text.strip()

        if text == "": #check to see if 
                                                #the text is blank
            widget.set_text(self.entry_old_value)
            return
        
        if text != self.entry_old_value:

            old_attrib = self.element.attrib.copy()

            result = meta_expr.is_valid(text,self.drop_list)
            if self.attrib == "name" or self.attrib == "draw_ops":
                self.element.attrib[self.attrib] = text
        
            elif result[0] == True:
                self.element.attrib[self.attrib] = text
            else:
                widget.set_text(self.entry_old_value)
                parent_window = widget.get_toplevel()
                dialogs.ErrorDialog(_("Error"),result[1],parent_window)
                return

            
            u = undo.AttribUndo(self.element,old_attrib)
            undo.add_undo(u)

            self.entry_old_value = text

            self.refresh()

    def check_callback(self,widget,data=None):
        
        old_attrib = self.element.attrib.copy()

        if widget.get_active():
            self.check.set_active(True)
            self.cEntry.set_sensitive(True)
            self.Entry.set_sensitive(True)
            self.label.set_sensitive(True)
            
            if self.user_set:
                self.element.attrib[self.attrib] = self.Entry.get_text()
        
        else:
            self.check.set_active(False)
            self.cEntry.set_sensitive(False)
            self.Entry.set_sensitive(False)
            self.label.set_sensitive(False)
            
            if self.user_set:
                self.element.attrib.pop(self.attrib)
        
        if self.user_set:
            u = undo.AttribUndo(self.element,old_attrib)
            undo.add_undo(u)

        self.refresh()

    def update(self):
        """ Update widgets when self.element changes"""

        self.user_set = False
        
        if self.element.attrib.has_key(self.attrib):
            if self.opt:
                self.check.set_active(True)
                self.cEntry.set_sensitive(True)
                self.Entry.set_sensitive(True)
                self.label.set_sensitive(True)
            
            if self.Entry.get_text() != self.element.attrib[self.attrib]:
                self.Entry.set_text(self.element.attrib[self.attrib])
        else:
            if self.opt:
                self.check.set_active(False)
            self.cEntry.set_sensitive(False)
            self.Entry.set_sensitive(False)
            self.label.set_sensitive(False)

        self.user_set = True
##############################################################################
### BoolBox
##############################################################################

class BoolBox(gtk.HBox):

    def __init__(self,refresh_func,element,attrib):
        
        gtk.HBox.__init__(self)

        self.element = element
        self.attrib = attrib

        
        self.refresh = refresh_func
        
        self.user_set = True

        label_text = attrib.split("_")
        label_text = " ".join(label_text)
        label = gtk.Label(label_text+":")
        self.label = label
        label.set_alignment(0.90,0.50)

        if len(label_text) > 10:
            label.set_size_request(120,-1)
        else:
            label.set_size_request(100,-1)

        self.pack_start(label,False,False)

        
        self.label.show()
        
        

        v = self.element.attrib[self.attrib]
        if v == "true":
            toggle = gtk.ToggleButton("Yes")

            toggle.set_active(True)

        else:
            toggle = gtk.ToggleButton("No")

            toggle.set_active(False)
            
        self.toggle = toggle
        
        toggle.connect("toggled",self.on_toggled)

        self.pack_start(toggle,True,True)

        self.toggle.show()
    
    
    def on_toggled(self,button,data=None):
        if not self.user_set:
            return

        old_attrib = self.element.attrib.copy()
        
        self.element.attrib[self.attrib] = ('false','true')[button.get_active()]
        self.toggle.set_label(('No','Yes')[button.get_active()])
            
        u = undo.AttribUndo(self.element,old_attrib)
        undo.add_undo(u)

        self.refresh()

    def update(self):
        """ Update widgets when self.element changes"""

        self.user_set = False
        
        if self.element.attrib[self.attrib] == "true":
            self.toggle.set_active(True)
            self.toggle.set_label("Yes")

        else:
            self.toggle.set_active(False)
            self.toggle.set_label("No")


        self.user_set = True

#####
#####
#####
class ColorBox(gtk.HBox):

    def __init__(self,refresh_func,element,attrib,drop_list=[],opt=False):
        
        gtk.HBox.__init__(self)

        self.element = element
        self.attrib = attrib

        
        self.refresh = refresh_func
        self.drop_list = drop_list
        self.opt = opt

        self.user_set = True

        if element.attrib.has_key(attrib):
            self.has_attr = True
        else:
            self.has_attr = False

        if opt:
            check = gtk.CheckButton(attrib+":")
            self.check = check
        
            label = check.get_child()
            self.label = label
        
            label_text = attrib.split("_")
            label_text = " ".join(label_text)
            label.set_text(label_text+":")

            label.set_alignment(0.90,0.50)
            if len(label_text) > 10:
                label.set_size_request(86,-1)
            else:
                label.set_size_request(76,-1)

            if self.has_attr:
                check.set_active(True)
            else:
                label.set_sensitive(False)
            
            check.connect("toggled",self.check_callback)
        
            self.pack_start(check,False,False)
            check.show()

        else:
            label_text = attrib.split("_")
            label_text = " ".join(label_text)
            label = gtk.Label(label_text+":")
            self.label = label
            label.set_alignment(0.90,0.50)

            label.set_size_request(100,-1)
            self.pack_start(label,False,False)

        label.show()

        cEntry = gtk.combo_box_entry_new_text()
        Entry = cEntry.get_children()[0]
        
        self.cEntry = cEntry
        self.Entry = Entry

        
        for text in drop_list:
            cEntry.append_text(text)

        self.entry_old_value = Entry.get_text()

        if self.has_attr:
            self.Entry.set_text(self.element.attrib[self.attrib])
            
        else:
            self.label.set_sensitive(False)
            self.cEntry.set_sensitive(False)

            v = opt_args[self.attrib]
            self.Entry.set_text(v)

        cEntry.connect("changed",self.on_cEntry_changed)

        Entry.connect("focus-in-event",self.on_focus_in)
        Entry.connect("focus-out-event",self.on_focus_out)
        Entry.connect("activate",self.on_activate)

        self.pack_start(cEntry,True,True)
        cEntry.show()
        
        ## add the color button
        self.colorbutton = gtk.ColorButton()

        self.colorbutton.connect("color-set",self.on_color_set)

        self.pack_start(self.colorbutton,False,False)

        self.colorbutton.show()
        
        if not self.has_attr:
            self.colorbutton.set_sensitive(False)
        
    
    def on_color_set(self,widget,data=None):
        color = widget.get_color()
        color = color.to_string()
        r,g,b = color[1:3],color[5:7],color[9:11]
        color = '#' + r + g + b
        #capitalize 'color'
        color = color.upper()

        self.Entry.set_text(color)
        self.Entry.activate()

    def on_cEntry_changed(self,widget,data=None):
        if widget.get_active() > -1:
            entry = widget.get_child()
            entry.activate()
    
        
    def on_focus_in(self,widget,event):
        self.entry_old_value = widget.get_text()

    def on_focus_out(self,widget,event):
        widget.activate()

    def on_activate(self,widget,data=None):
        text = widget.get_text()
        text = text.strip()

        if text == "" and not widget.is_focus(): #check to see if 
                                                #the text is blank
            widget.set_text(self.entry_old_value)
            return
        
        if text != self.entry_old_value:

            old_attrib = self.element.attrib.copy()

            result = meta_expr.is_valid(text,self.drop_list)
            if self.attrib == "name" or self.attrib == "draw_ops":
                self.element.attrib[self.attrib] = text
        
            elif result[0] == True:
                self.element.attrib[self.attrib] = text
            else:
                widget.set_text(self.entry_old_value)
                parent_window = widget.get_toplevel()
                dialogs.ErrorDialog(_("Error"),result[1],parent_window)
                return
            
            u = undo.AttribUndo(self.element,old_attrib)
            undo.add_undo(u)
            self.entry_old_value = text

            self.refresh()

    def check_callback(self,widget,data=None):
        
        if not self.user_set:
            return
        
        old_attrib = self.element.attrib.copy()

        if widget.get_active():
            self.check.set_active(True)
            self.cEntry.set_sensitive(True)
            self.Entry.set_sensitive(True)
            self.label.set_sensitive(True)
            self.colorbutton.set_sensitive(True)
            
            if self.user_set:
                self.element.attrib[self.attrib] = self.Entry.get_text()
        
        else:
            self.check.set_active(False)
            self.cEntry.set_sensitive(False)
            self.Entry.set_sensitive(False)
            self.label.set_sensitive(False)
            self.colorbutton.set_sensitive(False)
            
            if self.user_set:
                self.element.attrib.pop(self.attrib)
        
        u = undo.AttribUndo(self.element,old_attrib)

        undo.add_undo(u)

        self.refresh()

    def update(self):
        """ Update widgets when self.element changes"""
        
        self.user_set = False

        if self.element.attrib.has_key(self.attrib):
            if self.opt:
                self.check.set_active(True)
                self.cEntry.set_sensitive(True)
                self.label.set_sensitive(True)
            
            if self.Entry.get_text() != self.element.attrib[self.attrib]:
                self.Entry.set_text(self.element.attrib[self.attrib])
        else:
            if self.opt:
                self.check.set_active(False)
            self.cEntry.set_sensitive(False)
            self.label.set_sensitive(False)

        self.user_set = True

#####################################################################
#                                                                   #
#####################################################################

class FileBox(gtk.HBox):

    def __init__(self,refresh_func,element,dir,opt=False):
        
        gtk.HBox.__init__(self)

        self.element = element
        attrib = "filename"
        self.attrib = attrib
        self.dir = dir
        
        self.refresh = refresh_func
        self.opt = opt

        os.chdir(dir)
        file_list = os.listdir(dir)
        drop_list = []
        for file in file_list:
            if file.split('.')[-1] in ['png','PNG']:
                drop_list.append(file)

        self.drop_list = drop_list
        
        self.user_set = True

        if element.attrib.has_key(attrib):
            self.has_attr = True
        else:
            self.has_attr = False

        if opt:
            check = gtk.CheckButton(attrib+":")
            self.check = check
        
            label = check.get_child()
            self.label = label
        
            label_text = attrib.split("_")
            label_text = " ".join(label_text)
            label.set_text(label_text+":")

            label.set_alignment(0.90,0.50)
            if len(label_text) > 10:
                label.set_size_request(86,-1)
            else:
                label.set_size_request(76,-1)

            if self.has_attr:
                check.set_active(True)
            else:
                label.set_sensitive(False)
            
            check.connect("toggled",self.check_callback)
        
            self.pack_start(check,False,False)
            check.show()

        else:
            label_text = attrib.split("_")
            label_text = " ".join(label_text)
            label = gtk.Label(label_text+":")
            self.label = label
            label.set_alignment(0.90,0.50)

            label.set_size_request(100,-1)
            self.pack_start(label,False,False)

        
        self.label.show()

        Entry = gtk.Entry()

        self.Entry = Entry

        if self.has_attr:
            self.Entry.set_text(self.element.attrib[self.attrib])
            
        else:
            self.label.set_sensitive(False)
            self.Entry.set_sensitive(False)

            v = opt_args[self.attrib]
            self.Entry.set_text(v)

        self.entry_old_value = Entry.get_text()

        Entry.connect("focus-in-event",self.on_focus_in)
        Entry.connect("focus-out-event",self.on_focus_out)
        Entry.connect("activate",self.on_activate)

        self.pack_start(Entry,True,True)

        self.Entry.show()
        
        file_button = gtk.Button()
        image = gtk.Image()
        image.set_from_stock(gtk.STOCK_OPEN,2)
        file_button.add(image)
        image.show()
        file_button.connect("clicked",self.on_file_button_clicked)

        self.pack_start(file_button,False,False)

        file_button.show()

    
    def on_file_button_clicked(self,button,data=None):

        parent_window = button.get_toplevel()

        dialog = gtk.FileChooserDialog(_("Open..."),
            parent_window,
            gtk.FILE_CHOOSER_ACTION_OPEN,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        
        dialog.set_default_response(gtk.RESPONSE_OK)
        
        dialog.set_current_folder(self.dir)
        
        response = dialog.run()

        if response == gtk.RESPONSE_OK:
            selected = dialog.get_filename()
        else:
            selected = None

        dialog.destroy()
        
        if selected:
            dir,filename = os.path.split(selected)
            if dir != self.dir:
                new_filepath = utils.join(self.dir,filename)
                if os.path.isfile(new_filepath):
                    # If there is an existing file with the same
                    # name rename it so
                    # it doesn't get deleted
                    os.path.rename(new_filepath,filename+"1")
                
                output = commands.getoutput('cp "'+selected+'" "'+
                        self.dir+'"')

                utils.log(output)
                
            
            old_attrib = self.element.attrib.copy()

            
            file = selected.split('/')[-1]

            self.drop_list.append(file)
            
            self.Entry.set_text(file)
            self.element.attrib[self.attrib] = file

            u = undo.AttribUndo(self.element,old_attrib)
            undo.add_undo(u)

            self.refresh()
            

    def on_focus_in(self,widget,event):
        self.entry_old_value = widget.get_text()

    def on_focus_out(self,widget,event):
        widget.activate()

    def on_activate(self,widget,data=None):
        text = widget.get_text()

        if text == "" and not widget.is_focus(): #check to see if 
                                                #the text is blank
            widget.set_text(self.entry_old_value)
            return
        
        if text != self.entry_old_value:

            text = widget.get_text()
            text = text.strip()

            old_attrib = self.element.attrib.copy()

            result = meta_expr.is_valid(text,self.drop_list)
            if self.attrib == "name" or self.attrib == "draw_ops":
                self.element.attrib[self.attrib] = text
        
            elif result[0] == True:
                self.element.attrib[self.attrib] = text
            else:
                widget.set_text(self.entry_old_value)
                parent_window = widget.get_toplevel()
                dialogs.ErrorDialog(_("Error"),result[1],parent_window)

                return

            
            u = undo.AttribUndo(self.element,old_attrib)
            undo.add_undo(u)

            self.entry_old_value = text

            self.refresh()

    def check_callback(self,widget,data=None):
        
        old_attrib = self.element.attrib.copy()
        

        if widget.get_active():
            self.check.set_active(True)
            self.Entry.set_sensitive(True)
            self.label.set_sensitive(True)
            
            if self.user_set:
                self.element.attrib[self.attrib] = self.Entry.get_text()
        
        else:
            self.check.set_active(False)
            self.Entry.set_sensitive(False)
            self.label.set_sensitive(False)
            
            if self.user_set:
                self.element.attrib.pop(self.attrib)
        
        if self.user_set:
        
            u = undo.AttribUndo(self.element,old_attrib)
            undo.add_undo(u)

        self.refresh()
    
    
    def update(self):
        """ Update widgets when self.element changes"""

        self.user_set = False
        
        if self.element.attrib.has_key(self.attrib):
            if self.opt:
                self.check.set_active(True)
                self.Entry.set_sensitive(True)
                self.label.set_sensitive(True)

            self.Entry.set_text(self.element.attrib[self.attrib])
        else:
            if self.opt:
                self.check.set_active(False)
            self.Entry.set_sensitive(False)
            self.label.set_sensitive(False)

        self.user_set = True


####################################################################
#
####################################################################

class AttribComboBox(gtk.HBox):

    def __init__(self,refresh_func,element,attrib,drop_list,
            callback=False,opt=False):

        gtk.HBox.__init__(self)

        self.refresh = refresh_func
        self.element = element
        self.attrib = attrib
        self.drop_list = drop_list
        self.callback = callback
        self.opt = opt
    
        
        self.user_set = True

        if element.attrib.has_key(attrib):
            self.has_attr = True
        else:
            self.has_attr = False

        if opt:

            check = gtk.CheckButton(attrib+":")
            self.check = check

            if self.has_attr:
                check.set_active(True)

            label = check.get_child()
            self.label = label
            label.set_alignment(0.90,0.50)
            label.set_size_request(76,-1)

            start = [check.get_active()]
            
            if callback:
                check.connect("toggled",check_callback)
            else:
                check.connect("toggled",self.check_callback)

            self.pack_start(check,False,False)
            self.check.show()

        else:
            label = gtk.Label(attrib+":")
            self.label = label
            label.set_alignment(0.90,0.50)
            label.set_size_request(100,-1)

            self.pack_start(label,False,False)
            self.label.show()


        combobox = gtk.combo_box_new_text()
        self.combobox = combobox

        for text in drop_list:
            combobox.append_text(text)
    
        self.user_set = False

        if self.has_attr:
            combobox.set_active(drop_list.index(element.attrib[attrib]))
        else:
            combobox.set_active(len(self.drop_list)-1)

        self.user_set = True

        self.old_active_text = combobox.get_active_text()

        self.handler_id = combobox.connect("changed",self.on_combobox_changed)
    

        self.pack_start(combobox,True,True)

        combobox.show()
        
        if not self.has_attr:
            label.set_sensitive(False)
            combobox.set_sensitive(False)

    
    def set_element(self,element):
        self.element = element

        self.user_set = False

        self.combobox.set_active(self.drop_list.index(
            self.element.attrib[self.attrib]))

        self.user_set = True
    

    
    def on_combobox_changed(self,combobox,data=None):
        if not self.user_set:
            return

        text = combobox.get_active_text()
        if text == self.old_active_text:
            return

        self.old_active_text = text

        if self.callback:
            self.callback(combobox,self.element)
            return
         
        old_attrib = self.element.attrib.copy()
                
        self.element.attrib[self.attrib] = text
    
        u = undo.AttribUndo(self.element,old_attrib)
        undo.add_undo(u)
        
        self.refresh()

    def check_callback(self,widget,data=None):
        if not self.user_set:
            return
        
        old_attrib = self.element.attrib.copy()
        if widget.get_active():
            self.check.set_active(True)
            self.combobox.set_sensitive(True)
            self.label.set_sensitive(True)
            
            if self.user_set:
                self.element.attrib[self.attrib] = self.combobox.get_active_text()
        
        else:
            self.check.set_active(False)
            self.label.set_sensitive(False)
            self.combobox.set_sensitive(False)

            if self.user_set:
                self.element.attrib.pop(self.attrib)
        
        if self.user_set:
            u = undo.AttribUndo(self.element,old_attrib)
            undo.add_undo(u)

        self.refresh()

    def update(self):
        """ Update widgets when self.element changes"""

        self.user_set = False
        
        if self.element.attrib.has_key(self.attrib):
            if self.opt:
                self.check.set_active(True)
                self.combobox.set_sensitive(True)
                self.label.set_sensitive(True)
            

            self.combobox.disconnect(self.handler_id)

            model = self.combobox.get_model()
            model.clear()

            for text in self.drop_list:
                self.combobox.append_text(text)
            
            self.combobox.set_active(
                    self.drop_list.index(self.element.attrib[self.attrib]))
            
            self.old_active_text = self.element.attrib[self.attrib]

            self.handler_id = self.combobox.connect("changed",
                    self.on_combobox_changed)

        else:
            if self.opt:
                self.check.set_active(False)
            self.combobox.set_sensitive(False)
            self.label.set_sensitive(False)

        self.user_set = True

        
##############################################################################
### GradientBox
##############################################################################

class GradientBox(gtk.HBox):

    def __init__(self,refresh_func,element,consts):
        
        gtk.HBox.__init__(self)

        self.element = element
        
        self.refresh = refresh_func
        self.consts = consts
        
        self.color_list = []

        self.user_set = True

        label = gtk.Label(_("colors:"))
        self.label = label
        label.set_alignment(0.90,0.50)

        label.set_size_request(100,-1)
        self.pack_start(label,False,False)
        
        self.label.show()

        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)

        textview = gtk.TextView()
        self.textview = textview
        self.textbuffer = textview.get_buffer()
    
        sw.add(textview)

        self.textview.connect("focus-in-event",self.on_focus_in)
        
        
        text = []
        for child in element:
            text.append(child.attrib['value'])

        self.textbuffer.set_text("\n".join(text))

        self.old_text = self.get_text()

        self.pack_start(sw,True,True)
        sw.show_all()

        vbox = gtk.VBox()
    
        color_button = gtk.ColorButton()
        color_button.connect("color-set",self.on_color_button_clicked)


        color_button.show()

        vbox.pack_start(color_button,False,False)
    
        button_image = gtk.Image()
        button_image.set_from_stock(gtk.STOCK_APPLY,4)
        button_image.show()

        apply_button = gtk.Button()
        apply_button.add(button_image)
        apply_button.connect("clicked",self.on_apply_button_clicked)
        apply_button.show()

        vbox.pack_start(apply_button,False,False)
        vbox.show()

        self.pack_start(vbox,False,False)


    def on_color_button_clicked(self,widget,data=None):
        color = widget.get_color()
        color = color.to_string()
        r,g,b = color[1:3],color[5:7],color[9:11]
        color = '#' + r + g + b
        #capitalize 'color'
        color = color.upper()
        

        text = self.get_text()
        text = text+'\n'+color
        self.textbuffer.set_text(text)
        
        self.write_colors()
    
    def on_apply_button_clicked(self,button,data=None):
        
        self.write_colors()

    def get_text(self):
        start = self.textbuffer.get_start_iter()
        end = self.textbuffer.get_end_iter()
        text = self.textbuffer.get_text(start,end)
        return text

    
    def write_colors(self):
        text = self.get_text()
        
        if not text:
            self.textbuffer.set_text(self.old_text)
            return

        if text == self.old_text:
            return
        
        child_list = [child for child in self.element]
        
        num = 0
        last_color = None
        new_elems = []

        new_color_list = []

        for line in text.split("\n"):
            color = line.strip()
            if color:
                result = meta_expr.is_valid(color,self.consts)
                if result[0]:
                    new = ET.Element("color",{'value':color})
                    new_elems.append(new)
                    last_color = color

                    new_color_list.append(color)
                    num += 1
                else:
                    self.textbuffer.set_text(self.old_text)
                    parent_window = self.textview.get_toplevel()
                    dialogs.ErrorDialog(_("Error"),result[1],parent_window)

                    return
        
        if not last_color:
            self.textbuffer.set_text(self.old_text)
            return

        while num < 2:
            self.element.append(ET.Element("color",{'value':last_color}))
            text = text+"\n"+last_color
            num += 1
    
        # remove child elements
        for child in child_list:
            self.element.remove(child)

        # add new elements
        for child in new_elems:
            self.element.append(child)

        ## remove empty lines
        lines = []
        for line in text.split("\n"):
            lines.append(line.strip())
        
        text = "\n".join(lines)
        self.textbuffer.set_text(text)
        
        if new_color_list != self.color_list:
            u = undo.ChildrenUndo(self.element,child_list)
            undo.add_undo(u)

        self.old_text = text
        self.colors_list = new_color_list

        self.refresh()


    def on_focus_in(self,widget,event):
        self.color_list = []
        for line in self.get_text().split('\n'):
            color = line.strip()
            if color:
                result = meta_expr.is_valid(color,self.consts)
                if result[0]:
                    self.color_list.append(color)
             
        self.old_text = self.get_text()


    def update(self):
        """ Update widgets when self.element changes"""

        self.user_set = False
        col_list = []
        for child in self.element:
            col_list.append(child.attrib['value'])
        
        text = "\n".join(col_list)

        self.textbuffer.set_text(text)
        self.old_text = self.get_text()

        self.user_set = True

