#!/usr/bin/env python

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Authors: Isaiah Heyer (freshapplepy@gmail.com)

# Copyright (C) 2008 Isaiah Heyer

import gtk

def ErrorDialog(title,msg,parent_window):

    msg = str(msg)
    
    dialog = gtk.Dialog(title,
        parent_window,
        gtk.DIALOG_MODAL,
        (gtk.STOCK_CLOSE, gtk.RESPONSE_ACCEPT))
                
    dialog.set_default_response(gtk.RESPONSE_ACCEPT)
            
    dialog.set_default_size(200,100)
    hbox = gtk.HBox()

    dialog.vbox.pack_start(hbox,True,True)
            
    image = gtk.Image()
    image.set_from_stock(gtk.STOCK_DIALOG_ERROR,6)
            
    hbox.pack_start(image,True,True)

    label = gtk.Label(msg)

    hbox.pack_start(label,True,True)


    dialog.show_all()

    response = dialog.run()
        
    for s in [_("Negative value"),_("No constant named"),
            _("invalid syntax")]:

        if msg.startswith(s):
            break

    dialog.destroy()
    

def SaveDialog(theme_name,parent_window):

    dialog = gtk.Dialog(_("Save Theme"),
            parent_window,
            gtk.DIALOG_MODAL)
    
    dialog.set_has_separator(False)

    dialog.add_button(_("_Close without Saving"),gtk.RESPONSE_REJECT)
    dialog.add_button(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL)
    dialog.add_button(gtk.STOCK_SAVE,gtk.RESPONSE_OK)

    dialog.set_default_response(gtk.RESPONSE_CANCEL)
    dialog.set_default_size(200,100)

    hbox = gtk.HBox()

    dialog.vbox.pack_start(hbox,True,True)
            
    image = gtk.Image()
    image.set_from_stock(gtk.STOCK_DIALOG_WARNING,6)
            
    hbox.pack_start(image,True,True)

    lbox = gtk.VBox()
        
    hbox.pack_start(lbox,True,True)

    label = gtk.Label(
            '<span size="large"><b>'+_('Save changes to theme ')+
            '"'+theme_name+'"?</b></span>\n'+
            _(' Your changes will be lost if you don\'t save'))

    label.set_use_markup(True)

    lbox.pack_start(label,True,True)
        
    dialog.show_all()

    response = dialog.run()
        
    dialog.destroy()
    
    return response

def ConfirmDialog(title,msg,parent_window,default_response=False):
    """
    Dialog to ask the user for confirmation on 
    not undo-able actions.
    If default_reponse is True make the OK button be selected
    Return True if the user clicks ok and False otherwise
    """
    
    dialog = gtk.Dialog(title,
            parent_window,
            gtk.DIALOG_MODAL,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
            gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
                
    dialog.set_default_response(gtk.RESPONSE_CANCEL)
            
    dialog.set_default_size(200,100)
    hbox = gtk.HBox()

    dialog.vbox.pack_start(hbox,True,True)
            
    image = gtk.Image()
    image.set_from_stock(gtk.STOCK_DIALOG_WARNING,6)
            
    hbox.pack_start(image,True,True)

    label = gtk.Label(msg)
    label.set_use_markup(True)

    hbox.pack_start(label,True,True)


    dialog.show_all()

    response = dialog.run()
            
    if response == gtk.RESPONSE_ACCEPT:
        dialog.destroy()
        return True
    else:
        dialog.destroy()
        return False

def NameDialog(parent_window):
    dialog = gtk.Dialog(_("Set Name"),
            parent_window,
            gtk.DIALOG_MODAL,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
            gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
                
    dialog.set_default_response(gtk.RESPONSE_CANCEL)
            
    dialog.set_default_size(200,100)
    hbox = gtk.HBox()

    dialog.vbox.pack_start(hbox,True,True)
    
    entry = gtk.Entry()

    hbox.pack_start(entry,True,True)

    label = gtk.Label("...")
    label.set_use_markup(True)

    hbox.pack_start(label,True,True)


    dialog.show_all()

    response = dialog.run()
            
    if response == gtk.RESPONSE_ACCEPT:
        dialog.destroy()
        name = entry.get_text()
        return name
    else:
        dialog.destroy()
        return ""

def ConstantDialog(parent_window):
    dialog = gtk.Dialog(_("Set Name"),
            parent_window,
            gtk.DIALOG_MODAL,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
            gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
                
    dialog.set_default_response(gtk.RESPONSE_CANCEL)
            
    dialog.set_default_size(200,100)
    hbox = gtk.HBox()

    dialog.vbox.pack_start(hbox,True,True)
    
    label = gtk.Label(_("name: "))
    #label.set_use_markup(True)
    hbox.pack_start(label,True,True)

    n_entry = gtk.Entry()
    hbox.pack_start(n_entry,True,True)

    label = gtk.Label(_("value: "))
    #label.set_use_markup(True)
    hbox.pack_start(label,True,True)

    v_entry = gtk.Entry()
    hbox.pack_start(v_entry,True,True)

    
    dialog.show_all()

    response = dialog.run()
            
    if response == gtk.RESPONSE_ACCEPT:
        dialog.destroy()
        name = n_entry.get_text()
        value = v_entry.get_text()
        return name,value
    else:
        dialog.destroy()
        return "",""
