#!/usr/bin/env python

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Authors: Isaiah Heyer (freshapplepy@gmail.com)

# Copyright (C) 2008 Isaiah Heyer

from metacity_themer import GLADE_PATH, DB_FILENAME

from xml.etree import ElementTree as ET
import pygtk

pygtk.require('2.0')

import gtk
import os
import pickle

import attribs
import boxes
import undo
import dialogs
import utils

class Drawops(gtk.HBox):
    
    def __init__(self,refresh_func,TE):
            
        gtk.HBox.__init__(self)

        self.refresh = refresh_func
        
        self.TE = TE
        self.element = None
        
        self.can_paste = False

        self.user_set = True
        self.sel_child = None
        self.old_child_order = []
        
        self.builder = gtk.Builder()

        try:
            xml_file = utils.join(GLADE_PATH,"draw_ops.xml")
            self.builder.add_from_file(xml_file)

        except:
            print _("Failed to load: "),xml_file
                
        try:
            xml_file = utils.join(GLADE_PATH,"popup1.xml")
            self.builder.add_from_file(xml_file)

        except:
            print _("Failed to load: "),xml_file
         
        try:
            xml_file = utils.join(GLADE_PATH,"popup2.xml")
            self.builder.add_from_file(xml_file)

        except:
            print _("Failed to load: "),xml_file
         

        ##get widgets
        get = self.builder.get_object
        

        self.remove_button = get("remove_button")
        self.add_button = get("add_button")

        self.main_box = get("main_box")
        window = get("window")
        window.remove(self.main_box)
        self.prop_box = get("prop_box")

        ### treeview related widgets
        self.treeview = get("treeview")
        self.popup1 = get("popup_menu1")
        self.popup2 = get("popup_menu2")
        self.pm_paste = get("pm_paste")
        
        self.treeview.set_enable_search(False)
        selection = self.treeview.get_selection()
        selection.set_mode(gtk.SELECTION_MULTIPLE)

        self.treeview.connect("button-press-event",
                self.on_treeview_button_pressed)
        
        self.treeview.connect("key-press-event",
                self.on_treeview_key_pressed)
        
        self.type_cbb = get("type_combobox")
        
        ## add the main box

        self.add(self.main_box)

        #make treestore to show draw_ops element children
        self.liststore = gtk.ListStore(str,int)
        
        self.cell = gtk.CellRendererText()

        self.tvcolumn1 = gtk.TreeViewColumn("",self.cell)
        self.tvcolumn2 = gtk.TreeViewColumn("",self.cell)
        
        self.tvcolumn2.set_visible(False)

        self.tvcolumn1.add_attribute(self.cell,"text",0)
        self.tvcolumn2.add_attribute(self.cell,"text",1)

        self.treeview.append_column(self.tvcolumn1)
        self.treeview.append_column(self.tvcolumn2)
        
        self.treeview.set_model(self.liststore)
        ## enable sub element reordering
        self.treeview.set_reorderable(True)

        self.treeview.connect("drag-end",self.on_treeview_drag_end)

        #connect signals
        self.builder.connect_signals(self)

        #define some constants
            #the color constants defined in the gtk theme
        self.col_consts = ['gtk:fg[NORMAL]','gtk:fg[PRELIGHT]',
                    'gtk:fg[SELECTED]',
                'gtk:fg[ACTIVE]','gtk:fg[INSENSITIVE]',
                'gtk:bg[NORMAL]','gtk:bg[PRELIGHT]','gtk:bg[SELECTED]',
                'gtk:bg[ACTIVE]','gtk:bg[INSENSITIVE]',
                'gtk:base[NORMAL]','gtk:base[PRELIGHT]','gtk:base[SELECTED]',
                'gtk:base[ACTIVE]','gtk:base[INSENSITIVE]',
                'gtk:text[NORMAL]','gtk:text[PRELIGHT]','gtk:text[SELECTED]',
                'gtk:text[ACTIVE]','gtk:text[INSENSITIVE]']
        
        self.consts = ['left_width','right_width',
                'top_height','bottom_height','mini_icon_width',
                'mini_icon_height','icon_width','icon_height','title_width',
                'title_height','width','height']
        
        self.gtk_state_consts = ['normal','active','prelight','selected',
                'insensitive']

        self.gtk_shadow_consts = ['none','in','out','etched_in','etched_out']

        self.object_consts = ['object_width','object_height'] + self.consts[:]
        
        
        #used for indexing self.type_cbb:
        self.types = ['arc','clip','gradient','gtk_arrow','gtk_box',
                'gtk_vline','icon','image','include','line','rectangle',
                'tile','tint','title']
    

    def set_element(self,element):
        self.user_set = False

        self.element = element
        
        self.sel_child = None

        self.load_children()

        self.select_none()

        self.user_set = True

        if self.element.attrib['name'] == "~FOO_do~":
            self.main_box.set_sensitive(False)
        else:
            self.main_box.set_sensitive(True)

    def select_none(self):

        self.type_cbb.set_active(-1)
        self.type_cbb.set_sensitive(False)

        for child in self.prop_box.get_children():
            self.prop_box.remove(child)

        self.sel_child = None

    
    def update(self):
        """ 
        update all widgets, call after undo/redo
        """
        
        self.load_children() 

        if self.sel_child != None:
            
            i = self.type_cbb.get_active()
            sel_text = self.types[i]

            if sel_text != self.sel_child.tag:
                self.load_properties()

        pthinfo = self.treeview.get_path_at_pos(self.treeview_x,
            self.treeview_y)

        if pthinfo is not None:
            path, col, cellx, celly = pthinfo
            self.treeview.grab_focus()
            self.treeview.set_cursor( path, col, 0)

        for child in self.prop_box.get_children():
            if type(child) != gtk.HBox:
                child.update()
        
        

    def load_children(self):
        k = 0
        self.ops = {} #dictionary to store the draw_ops children elements
        self.treeview.set_model(None)
        self.liststore.clear()

        for child in self.element:
            
            self.liststore.append([child.tag,k])

            self.ops[k] = child

            k += 1
        
        self.treeview.set_model(self.liststore)
    
    def copy(self):

        selection = self.treeview.get_selection()
        model, paths = selection.get_selected_rows()
        
        keys = []

        for p in paths:
            Iter = model.get_iter(p)
            keys.append(model.get_value(Iter,1))

        new_elems = []

        for key in keys:
            elem = self.ops[key]
            elem_children = [child for child in elem]
            new_elem = ET.Element(elem.tag,elem.attrib.copy())
            for child in elem_children:
                new_elem.append(ET.Element(child.tag,child.attrib.copy()))
            
            new_elems.append(elem)
        
        utils.copy_elems(new_elems)
        self.can_paste = True

    
    def paste(self):

        child_list = [child for child in self.element]  

        for elem in utils.paste_elems():
            if elem.tag == "include" or elem.tag == "tile":
                if not self.TE.is_valid_ref(elem,self.element):

                    dialogs.ErrorDialog(_("Error"),
                            _("Can not paste this element here"),
                            self.get_toplevel())

                    return False

            elif elem.tag == "image":
                if not elem.attrib['filename'] in os.listdir(self.TE.get_dir()):
                    
                    dialogs.ErrorDialog(_("Error"),
                            _("Can not paste this element here,\n"+
                                "image file not found"),
                            self.get_toplevel())

                    return False


            self.element.append(elem)
            k = len(self.element.getchildren())
            self.ops[k] = elem
            self.liststore.append([elem.tag,k])
        
        new_child_list = [child for child in self.element]

        if child_list != new_child_list:
            u = undo.ChildrenUndo(self.element,child_list)
            undo.add_undo(u)

        self.refresh()

    def on_treeview_drag_end(self,treeview,context):

        child_order = [child for child in self.element]

        model = treeview.get_model()
        ## clear self.element
        for child in child_order:
            self.element.remove(child)
        
        ## add reorder it
        for row in model:
            self.element.append(self.ops[row[1]])
        
        self.load_children()
        
        u = undo.ChildrenUndo(self.element,child_order)

        if child_order != self.old_child_order:
            undo.add_undo(u)
        
        self.old_child_order = child_order

            
        #refresh the preview
        self.refresh()
    
    def on_treeview_cursor_changed(self,widget,data=None):

        selected = self.treeview.get_selection()
        model,paths = selected.get_selected_rows()
            
        if not paths:
            return

        Iter = model.get_iter(paths[0])

        k = self.liststore.get_value(Iter,1)
        self.sel_child = self.ops[k]
    
        self.load_properties()
    
    def on_treeview_button_pressed(self,treeview,event):

        self.treeview_x = int(event.x)
        self.treeview_y = int(event.y)

        if event.button == 3:

            time = event.time
            pthinfo = treeview.get_path_at_pos(self.treeview_x,
                    self.treeview_y)

            if pthinfo is not None:
                path, col, cellx, celly = pthinfo
                treeview.grab_focus()
                treeview.set_cursor( path, col, 0)
                self.popup1.popup( None, None, None, event.button, time)

            else:
                treeview.grab_focus()
                if utils.get_can_paste():
                    self.pm_paste.set_sensitive(True)
                else:
                    self.pm_paste.set_sensitive(False)

                self.popup2.popup( None, None, None, event.button, time)

            return True

    def on_treeview_key_pressed(self,treeview,event):

        ## copy and paste

        if event.keyval == 99:
            if event.state == gtk.gdk.CONTROL_MASK | gtk.gdk.MOD2_MASK:
                self.copy()

        if event.keyval == 118:
            if event.state == gtk.gdk.CONTROL_MASK | gtk.gdk.MOD2_MASK:
                if utils.get_can_paste():
                    self.paste()
        #####

    def on_pm_remove_activate(self,widget,data=None):
        if self.sel_child == None:
            return

        child_list = [child for child in self.element]

        self.element.remove(self.sel_child)

        u = undo.ChildrenUndo(self.element,child_list)

        undo.add_undo(u)

        self.load_children()
        self.select_none()
        
        self.refresh()
    

    def on_pm_copy_activate(self,widget,data=None):
        self.copy()


    def on_pm_paste_activate(self,widget,data=None):
        self.paste()
        

    ###### utility functions
    def add_file_box(self,attrib,optional=False):
        new_box = boxes.FileBox(self.refresh,self.sel_child,
                self.TE.get_dir(),optional)
        
        self.prop_box.pack_start(new_box,False,False)
        new_box.show()

    def add_box(self,attrib,drop_list=[],optional=False):
        new_box = boxes.AttribBox(self.refresh,self.sel_child,
                attrib,drop_list,optional)
            
        self.prop_box.pack_start(new_box,False,False)
        new_box.show()

        
    def add_color_box(self,attrib,drop_list=[],optional=False):
        new_box = boxes.ColorBox(self.refresh,self.sel_child,
                attrib,drop_list,optional)
        
        self.prop_box.pack_start(new_box,False,False)
        new_box.show()

    def add_bool_box(self,attrib):
        element = self.sel_child
        new_box = boxes.BoolBox(self.refresh,element,attrib)
        
        self.prop_box.pack_start(new_box,False,False)
        new_box.show()
        
    def add_gradient_box(self):
        element = self.sel_child
        new_box = boxes.GradientBox(self.refresh,element,
            self.col_consts+self.consts+self.TE.get_constant_list())
        
        self.prop_box.pack_start(new_box,False,False)
        new_box.show()
        
    def add_cbb_box(self,attrib,drop_list=[]):
        element = self.sel_child

        self.prop_box.pack_start(boxes.AttribComboBox(self.refresh,
            element,attrib,drop_list),False,False)

    def load_properties(self):
        
        self.prop_box.hide()
        
        consts = self.consts + self.TE.get_constant_list()
        object_consts = self.object_consts + self.TE.get_constant_list()

        for row in self.liststore:
            row[0] = self.ops[row[1]].tag

        value = self.sel_child.tag

        i = self.type_cbb.get_active()
        sel_text = self.types[i]

        if sel_text != self.sel_child.tag:
            self.user_set = False
                
            self.type_cbb.set_active(self.types.index(
                self.sel_child.tag))

            self.user_set = True


        for child in self.prop_box.get_children():
            self.prop_box.remove(child)
        

        if value == "arc":
            self.add_color_box("color",self.col_consts)
            self.add_box('x',drop_list=consts)            
            self.add_box('y',drop_list=consts)
            self.add_box('width',drop_list=consts)
            self.add_box('height',drop_list=consts)
            self.add_box('start_angle')
            self.add_box('extent_angle')
            self.add_bool_box('filled')
            
            
        elif value == "clip":
            self.add_box('x',drop_list=consts)
            self.add_box('y',drop_list=consts)
            self.add_box('width',drop_list=consts)
            self.add_box('height',drop_list=consts)
        
        elif value == "gradient":
            self.add_gradient_box()
            self.add_box('x',drop_list=consts)
            self.add_box('y',drop_list=consts)
            self.add_box('width',drop_list=consts)
            self.add_box('height',drop_list=consts)

        elif value == "gtk_arrow":
            self.add_cbb_box('state',drop_list=self.gtk_state_consts)
            self.add_cbb_box('shadow',drop_list=self.gtk_shadow_consts)
            self.add_cbb_box('arrow',drop_list=['left','right','up','down'])
            self.add_box('x',drop_list=consts)
            self.add_box('y',drop_list=consts)
            self.add_box('width',drop_list=consts)
            self.add_box('height',drop_list=consts)
            self.add_bool_box('filled')
        
        elif value == "gtk_box":
            self.add_cbb_box('state',drop_list=self.gtk_state_consts)
            self.add_cbb_box('shadow',drop_list=self.gtk_shadow_consts)
            self.add_box("x",drop_list=consts)
            self.add_box("y",drop_list=consts)
            self.add_box("width",drop_list=consts)
            self.add_box("height",drop_list=consts)
        
        elif value == "gtk_vline":
            self.add_cbb_box('state',drop_list=self.gtk_state_consts)
            self.add_box("x",drop_list=consts)
            self.add_box("y1",drop_list=consts)
            self.add_box("y2",drop_list=consts)
        
        elif value == "icon":
            self.add_box('x',drop_list=consts)
            self.add_box('y',drop_list=consts)
            self.add_box('width',drop_list=consts)
            self.add_box('height',drop_list=consts)
            self.add_box('alpha',optional=True)

   
        elif value == "image":
            self.add_file_box('filename')
            self.add_box('x',drop_list=object_consts)            
            self.add_box('y',drop_list=object_consts)
            self.add_box('width',drop_list=object_consts)
            self.add_box('height',drop_list=object_consts)
            self.add_box('alpha',optional=True)
            self.add_color_box('colorize',self.col_consts,True)
        
        elif value == "include":
            include_list = self.TE.get_includable(self.element.attrib['name'])
            self.add_box('name',drop_list=include_list)

            self.add_box('x',drop_list=object_consts,optional=True)
            self.add_box('y',drop_list=object_consts,optional=True)
            self.add_box('width',drop_list=object_consts,optional=True)
            self.add_box('height',drop_list=object_consts,optional=True)

        elif value == "line":
            self.add_color_box('color',self.col_consts)
            self.add_box('x1',drop_list=consts)            
            self.add_box('y1',drop_list=consts)
            self.add_box('x2',drop_list=consts)            
            self.add_box('y2',drop_list=consts)
            self.add_box('width',optional=True)
            self.add_box('dash_on_length',optional=True)
            self.add_box('dash_off_length',optional=True)

        elif value == "rectangle":
            self.add_color_box('color',self.col_consts)
            self.add_box('x',consts)
            self.add_box('y',consts)
            self.add_box('width',consts)
            self.add_box('height',consts)
            self.add_bool_box('filled')

        elif value == "tile":
            include_list = self.TE.get_includable(self.element.attrib['name'])
            self.add_box('name',drop_list=include_list)

            self.add_box('tile_width',consts)
            self.add_box('tile_height',consts)
            self.add_box('x',consts,optional=True)
            self.add_box('y',consts,optional=True)
            self.add_box('width',consts,optional=True)
            self.add_box('height',consts,optional=True)
            self.add_box('tile_xoffset',consts,optional=True)
            self.add_box('tile_yoffset',consts,optional=True)
        
        elif value == "tint":
            self.add_color_box('color',self.col_consts)
            self.add_box('x',consts)
            self.add_box('y',consts)
            self.add_box('width',consts)
            self.add_box('height',consts)
            self.add_box('alpha',consts)
   
        elif value == "title":
            self.add_color_box('color',self.col_consts)
            self.add_box('x',consts)            
            self.add_box('y',consts)
        
        
        #make the type_cbb sensitive
        self.type_cbb.set_sensitive(True)
           
        self.prop_box.show()
        
    def on_type_combobox_changed(self,widget,data=None):
        
        if not self.user_set:
            return

        index = widget.get_active()
        if index == -1:
            return False

        
        old_tag = self.sel_child.tag
        old_attrib = self.sel_child.attrib.copy()
        old_child_list = [child for child in self.sel_child]
        
        i = widget.get_active()
        sel_text = self.types[i]

        if sel_text != self.sel_child.tag:
            if sel_text == "gradient":
                self.sel_child.tag = sel_text
                self.sel_child.attrib = attribs.get_attribs(sel_text)
                # add the color sub elements
                color = ET.Element("color",{"value":"#000000"})
                self.sel_child.append(color)
                self.sel_child.append(color)

                self.load_properties()

            else:
                ##remove any child elements if it has any
                rem_list = [child for child in self.sel_child]
                for child in rem_list:
                    self.sel_child.remove(child)

                self.sel_child.tag = sel_text
                self.sel_child.attrib = attribs.get_attribs(sel_text)
                
                self.load_properties()
                
        u = undo.TypeUndo(self.sel_child,old_tag,old_attrib,old_child_list)
        undo.add_undo(u)

        self.refresh()

        
    def on_add_button_clicked(self,widget,data=None):
        new = ET.Element("line",
            {'color':"#000000",'x1':'0','y1':'0','x2':'0','y2':'0'})
        
        child_list = [child for child in self.element]

        self.element.append(new)

        u = undo.ChildrenUndo(self.element,child_list)

        undo.add_undo(u)
        

        self.load_children()
        self.select_none()

        self.refresh()


    def on_remove_button_clicked(self,widget,data=None):
        if self.sel_child != None:
            child_list = [child for child in self.element]

            self.element.remove(self.sel_child)

            u = undo.ChildrenUndo(self.element,child_list)

            undo.add_undo(u)


            self.load_children()
            self.select_none()
            
            self.refresh()

class FrameGeometry(gtk.VBox):

    def __init__(self,refresh,elem):
        
        gtk.VBox.__init__(self)
        
        
        geometry = ['title_scale','has_title','rounded_top_left',
                        'rounded_top_right','rounded_bottom_left',
                        'rounded_top_right']
        
        sub_geometry = ['left_width','right_width','bottom_height',
                'left_titlebar_edge','right_titlebar_edge',
                'title_vertical_pad','title_border']


        self.update_boxes = []
        for g in geometry:

            label = gtk.Label("<b>"+g+"</b>")
            label.set_use_markup(True)
            self.pack_start(label,False,False)
                        
            if g == "title_scale":

                box = boxes.AttribComboBox(refresh,elem,g,
                        ["xx-small","x-small","small","medium",
                        "large","x-large","xx-large"],opt=True)

                         
            else:
                if not elem.attrib.has_key(g):
                    if g == "has_title":
                        elem.attrib[g] = "true"
                    else:
                        elem.attrib[g] = "false"

                box = boxes.BoolBox(refresh,elem,g)
                                    
            self.pack_start(box,False,False)
            self.update_boxes.append(box)

                        
        for g in sub_geometry:
            for child in elem:
                if child.attrib['name'] == g:
                    label = gtk.Label("<b>"+g+"</b>")
                    label.set_use_markup(True)
                    self.pack_start(label,False,False)

                    if len(child.attrib.keys()) > 2:
                        for dir in ['left','right','top','bottom']:
                            
                            box = boxes.AttribBox(refresh,child,
                                    dir)

                            self.pack_start(box,False,False)
                            self.update_boxes.append(box)
                    else:
                        
                        box = boxes.AttribBox(refresh,child,
                                "value")
 
                        self.pack_start(box,False,False)
                        self.update_boxes.append(box)

    def update(self):
        for box in self.update_boxes:
            box.update()

class ButtonGeometry(gtk.VBox):
        
    def __init__(self,refresh,elem):

        gtk.VBox.__init__(self)
       
        self.refresh = refresh
        self.elem = elem
        
        self.user_set = True
        
        self.update_boxes = []

        hbox = gtk.HBox()
        self.pack_start(hbox,True,True,5)

        left_box = gtk.VBox()
        hbox.pack_start(left_box,False,False)

        right_box = gtk.VBox()
        hbox.pack_start(right_box,True,True,5)
                
        viewport1 = gtk.Viewport()
        viewport2 = gtk.Viewport()
        
        box1 = gtk.VBox()
        box2 = gtk.VBox()
        
        self.box1 = box1
        self.box2 = box2

        viewport1.add(box1)
        viewport2.add(box2)

        right_box.pack_start(viewport1,True,True,5)
        right_box.pack_start(viewport2,True,True,5)

        radio_button1 = gtk.RadioButton(label="Width/Height")
        self.radio_button1 = radio_button1             

        radio_button2 = gtk.RadioButton(group=radio_button1,
            label="Aspect Ratio")

        self.radio_button2 = radio_button2

        left_box.pack_start(radio_button1,True,False)
        left_box.pack_start(radio_button2,True,False)

        aspect_ratio = False
        width_height = False
        for n in ['button_width','button_height']:
            for child in elem:
                if child.attrib['name'] == n:
                    width_height = True
                    if n == "button_width":
                        self.button_width_elem = child
                    else:
                        self.button_height_elem = child

                    label = gtk.Label("<b>"+" ".join(n.split("_"))+"</b>")
                    label.set_use_markup(True)
                    label.set_alignment(0.01,0.60)

                    box1.pack_start(label,False,False)

                    box = boxes.AttribBox(self.refresh,child,
                            "value")

                    box1.pack_start(box,False,False)
                    self.update_boxes.append(box)

                    radio_button1.set_active(True)

        for child in elem:
            if child.tag == "aspect_ratio":
                aspect_ratio = True
                self.aspect_ratio_elem = child

                label = gtk.Label("<b>"+" ".join(n.split("_"))+"</b>")
                label.set_use_markup(True)
                label.set_alignment(0.01,0.60)

                box2.pack_start(label,False,False)

                box = boxes.AttribBox(self.refresh,child,"value")
                
                box2.pack_start(box,False,False)
                self.update_boxes.append(box)
                
                radio_button2.set_active(True)

        if not width_height:
            
            box1.set_sensitive(False)

            self.button_width_elem = ET.Element("distance",
                    {'name':'button_width','value':'10'})

            label = gtk.Label("<b>button width</b>")
            label.set_use_markup(True)
            label.set_alignment(0.01,0.60)

            box1.pack_start(label,False,False)


            box = boxes.AttribBox(self.refresh,
                    self.button_width_elem,"value")

            box1.pack_start(box,False,False)
        
            ###########################
            self.button_height_elem = ET.Element("distance",
                    {'name':'button_height','value':'10'})
            
            label = gtk.Label("<b>button height</b>")
            label.set_use_markup(True)
            label.set_alignment(0.01,0.60)

            box1.pack_start(label,False,False)


            box = boxes.AttribBox(self.refresh,self.button_height_elem,
                    "value")

            box1.pack_start(box,False,False)
            self.update_boxes.append(box)

        elif not aspect_ratio:
            box2.set_sensitive(False)

            self.aspect_ratio_elem = ET.Element("aspect_ratio",
                    {'name':'button','value':'1.0'})

            label = gtk.Label("<b>aspect ratio</b>")
            label.set_use_markup(True)
            label.set_alignment(0.01,0.60)

            box2.pack_start(label,False,False)


            box = boxes.AttribBox(self.refresh,self.aspect_ratio_elem,
                    "value")

            box2.pack_start(box,False,False)
            self.update_boxes.append(box)


        radio_button1.connect("toggled",self.on_toggled)
        
        border_box = gtk.VBox()
        
        label = gtk.Label("<b>button border</b>")
        label.set_use_markup(True)
        label.set_alignment(0.01,0.60)
        
        border_box.pack_start(label,False,False)

        hsep = gtk.HSeparator()

        border_box.pack_start(hsep,False,False)

        for child in elem:
            if child.attrib['name'] == "button_border":
                for dir in ['left','right','top','bottom']:
                    label = gtk.Label("<b>"+dir+"</b>")
                    label.set_use_markup(True)

                    border_box.pack_start(label,False,False)
                    
                    box = boxes.AttribBox(self.refresh,child,dir)
         
                    border_box.pack_start(box,False,False)
                    self.update_boxes.append(box)

        self.pack_start(border_box,True,True)


    def on_toggled(self,button,data=None):
        if not self.user_set:
            return
        
        child_list = [child for child in self.elem]

        is_active = button.get_active()
        if is_active:
            self.elem.remove(self.aspect_ratio_elem)

            self.elem.append(self.button_width_elem)
            self.elem.append(self.button_height_elem)
                
            self.box1.set_sensitive(True)
            self.box2.set_sensitive(False)
        else:
            self.elem.remove(self.button_width_elem)
            self.elem.remove(self.button_height_elem)

            self.elem.append(self.aspect_ratio_elem)
                
            self.box1.set_sensitive(False)
            self.box2.set_sensitive(True)

        u = undo.ChildrenUndo(self.elem,child_list)
        
        undo.add_undo(u)
        
        self.refresh()
    
    def update(self):
        
        self.user_set = False

        for box in self.update_boxes:
            box.update()
        
        aspect_ratio = False

        for child in self.elem:
            if child.tag == "aspect_ratio":
                aspect_ratio = True
                break

        if aspect_ratio:
            self.radio_button2.set_active(True)
            self.box1.set_sensitive(False)
            self.box2.set_sensitive(True)

        else:
            self.radio_button1.set_active(True)
            self.box1.set_sensitive(True)
            self.box2.set_sensitive(False)

        self.user_set = True

class Frame(gtk.VBox):

    def __init__(self,refresh_func,win_type,frame_elem,TE):
        
        gtk.VBox.__init__(self)

        self.refresh = refresh_func
        self.element = frame_elem
            
        self.win_type = win_type
        self._state = self.element.attrib['state']
        self._focus = self.element.attrib['focus']
        self.TE = TE
        

        fs_list = self.TE.get_fs_list()
        
        label_text = self._state
        label_text.capitalize()

        focus = self._focus
        if focus == "yes":
            label_text += " Focused"
        else:
            label_text += " Unfocused"

        label = gtk.Label("<b><span size='x-large'>"+label_text+"</span></b>")
        label.set_use_markup(True)
        label.set_alignment(0.10,0.50)

        self.pack_start(label,False,False)

        label.show()

        self.combobox = boxes.AttribComboBox(refresh_func,self.element,
                'style',fs_list,self.on_combobox_changed)
    
        self.pack_start(self.combobox,False,True)
        
        self.combobox.show()
        
        hsep = gtk.HSeparator()
        self.pack_start(hsep,False,False)
        hsep.show()

        self.fs_area = gtk.HBox()

        self.pack_start(self.fs_area)
        
        frame_style_name = self.element.attrib['style']
        frame_style = self.TE.frame_style[frame_style_name]
    
        self.FSBox = FrameStyle(self.refresh,frame_style,self.TE)
        self.fs_area.add(self.FSBox)

        self.fs_area.show_all()

    def on_combobox_changed(self,combobox,element):
        
        old_attrib = element.attrib.copy()

        text = combobox.get_active_text()
        self.TE.set_frame_style(self.win_type,element,text)
        self.TE.set_preview_state(self.win_type,self._focus,self._state)    
        self.refresh()

        u = undo.FrameUndo(self.TE,self.win_type,element,old_attrib)
        undo.add_undo(u)
        
        ### update the FrameStyle Box
        self.fs_area.remove(self.FSBox)

        frame_style_name = self.element.attrib['style']
        frame_style = self.TE.frame_style[frame_style_name]

        self.FSBox = FrameStyle(self.refresh,frame_style,self.TE)
        
        self.fs_area.add(self.FSBox)
        self.fs_area.show_all()

    def update(self):
        self.FSBox.update()
        self.combobox.drop_list = self.TE.get_fs_list()
        self.combobox.update()

class FrameStyle(gtk.VBox):

    def __init__(self,refresh_func,element,TE):

        gtk.VBox.__init__(self)
        
        self.refresh = refresh_func
        self.element = element
        self.TE = TE
        
        label_text = ("<b>Frame Style: "+element.attrib['name']+"</b>")
        
        label = gtk.Label(label_text)
        label.set_use_markup(True)

        self.pack_start(label,False,False)
        label.show()

        fg_list = self.TE.get_geometry_list()
        
        self.combobox = boxes.AttribComboBox(refresh_func,self.element,
                'geometry',fg_list,self.on_combobox_changed)
    
        self.pack_start(self.combobox,False,True)

        self.combobox.show()

    def on_combobox_changed(self,combobox,element):
        
        old_attrib = element.attrib.copy()

        text = combobox.get_active_text()
        element.attrib['geometry'] = text

        u = undo.AttribUndo(element,old_attrib)
        undo.add_undo(u)
        
        self.refresh()
    
    def update(self):
        self.combobox.drop_list = self.TE.get_geometry_list()
        self.combobox.update()

class Window(gtk.VBox):

    def __init__(self,refresh_func,element,TE):

        gtk.VBox.__init__(self)

        self.refresh = refresh_func
        self.element = element
        self.TE = TE
        
        self.win_type = self.element.attrib['type']

        #label_text = ("<b>"+"Frame Style Set"+"</b>")
        
        #label = gtk.Label(label_text)
        #label.set_use_markup(True)

        #self.pack_start(label,False,False)
        #label.show()

        fss_list = self.TE.get_fss_list()
        
        self.combobox = boxes.AttribComboBox(refresh_func,self.element,
                'style_set',fss_list,self.on_combobox_changed)
    
        self.pack_start(self.combobox,False,True)

        self.combobox.show()

    def on_combobox_changed(self,combobox,element):
        
        old_attrib = element.attrib.copy()

        text = combobox.get_active_text()
        self.TE.set_frame_style_set(element,text)

        ## set the preview state
        self.TE.set_preview_state(self.win_type,"yes","normal")
        self.refresh()

        u = undo.WindowUndo(self.TE,element,old_attrib)
        undo.add_undo(u)
    
    def update(self):
        self.combobox.drop_list = self.TE.get_fss_list()
        self.combobox.update()

