#!/usr/bin/env python

#This program and all files included in this package are free software;
#you can redistribute it and/or modify it under the terms
#of the GNU General Public License as published by the Free
#Software Foundation; either version 2 of the License,
#or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful, 
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from metacity_themer import GLADE_PATH

import pygtk
pygtk.require('2.0')
import gtk

import os
import dialogs
import undo
import utils

class Editor(object):

    def __init__(self,TE,parent_window,refresh_func):

        self.TE = TE
        self.refresh = refresh_func
        
        self.undo_list = []
        self.redo_list = []

        self.builder = gtk.Builder()
        try:
            xml_file = os.path.join(GLADE_PATH,"elementsgui.xml")
            self.builder.add_from_file(xml_file)

        except Exception,data:
            utils.log("Failed to load "+xml_file)
            print "Failed to load "+xml_file

        ### get widgets

        ### add the main box to self
        get = self.builder.get_object

        self.window = get("window")
        self.window.connect("delete_event",self.on_delete_event)
        self.window.set_modal(True)
        self.window.set_transient_for(parent_window)

        main_box = get("main_box")

        ## now for the rest of the widgets
        self.ops_undo = get("ops_undo")
        self.ops_redo = get("ops_redo")
        self.close_button = get("close_button")

        ## drawops tab widgets:
        self.ops_add = get("ops_add")
        self.ops_remove = get("ops_remove")
        self.ops_treeview = get("ops_treeview")
        self.ops_treeview.connect("drag-begin",self.on_drag_begin)
        self.ops_treeview.connect("drag-end",self.on_drag_end)
        self.ops_treeview.set_reorderable(True)

        self.ops_label = get("ops_label")

        ## geometry tab widgets:
        self.geo_add = get("geo_add")
        self.geo_remove = get("geo_remove")
        self.geo_treeview = get("geo_treeview")
        self.geo_label = get("geo_label")

        ## frame style tab widgets:
        self.style_add = get("style_add")
        self.style_remove = get("style_remove")
        self.style_treeview = get("style_treeview")
        self.style_label = get("style_label")
        
        ## frame style set tab widgets:
        self.set_add = get("set_add")
        self.set_remove = get("set_remove")
        self.set_treeview = get("set_treeview")
        self.set_label = get("set_label")
        
        ## constant tab widgets:
        self.con_add = get("con_add")
        self.con_remove = get("con_remove")
        self.con_treeview = get("con_treeview")
        self.con_label = get("con_label")
        

        ### initialize draw ops tab
        self.treeview_build(self.ops_treeview,self.TE.get_drawops_list())

        ### initialize geometry tab
        self.treeview_build(self.geo_treeview,self.TE.get_geometry_list())

        ### initialize frame style tab
        self.treeview_build(self.style_treeview,self.TE.get_frame_style_list())

        ### initialize frame style set tab
        self.treeview_build(self.set_treeview,
                self.TE.get_frame_style_set_list())
        
        ### initialize constants tab
        self.treeview_build(self.con_treeview,self.TE.get_constant_list())

        self.window.show_all()
        self.builder.connect_signals(self)
    
    def on_delete_event(self,window,data=None):
        window.hide_on_delete()
        return True

    def on_close_button_clicked(self,button,data=None):
        self.window.hide()

    def treeview_build(self,treeview,elem_list):
        
        cell = gtk.CellRendererText()
        tvcolumn = gtk.TreeViewColumn("",cell)
        tvcolumn.add_attribute(cell,"text",0)
        
        treeview.append_column(tvcolumn)

        model = gtk.ListStore(str)

        for name in elem_list:
            model.append((name,))
        
        treeview.set_model(model)
        treeview.set_headers_visible(False)
    
    
    def on_remove_button_clicked(self,button,type):

        if type == "ops":
            selection = self.ops_treeview.get_selection()
            model, Iter = selected.get_selected()
            
            name = model.get_value(Iter,0)
            
            old_child_list = self.TE.get_all_elements()

            self.TE.remove_element(name,"draw_ops")

            model.remove(Iter)
            
            undo.ChildrenUndo(self.TE.root,old_child_list)
    
    def add_element(self,name,type,value=None):

        if type == "draw_ops":
            self.TE.make_drawops(name)
            model = self.ops_treeview.get_model()
            model.append([name])

        elif type == "frame_geometry":
            self.TE.make_geometry(name)
            model = self.geo_treeview.get_model()
            model.append([name])
        
        elif type == "frame_style":
            self.TE.make_frame_style(name)
            model = self.style_treeview.get_model()
            model.append([name])

        elif type == "frame_style_set":
            self.TE.make_frame_style_set(name)
            model = self.set_treeview.get_model()
            model.append([name])

        elif type == "constant":
            self.TE.make_constant(name,value)
            model = self.con_treeview.get_model()
            model.append([name])

    def on_ops_treeview_cursor_changed(self,treeview,data=None):
        selection = treeview.get_selection()
        model,iter = selection.get_selected()
        
        if iter != None:
            name = model.get_value(iter,0)
        
            if name == "~FOO_do~":
                self.ops_remove.set_sensitive(False)
            else:
                self.ops_remove.set_sensitive(True)

    def on_geo_treeview_cursor_changed(self,treeview,data=None):
        selection = treeview.get_selection()
        model,iter = selection.get_selected()
        
        if iter != None:
            name = model.get_value(iter,0)
        
            if name == "~FOO_fg~":
                self.geo_remove.set_sensitive(False)
            else:
                self.geo_remove.set_sensitive(True)

    def on_style_treeview_cursor_changed(self,treeview,data=None):
        selection = treeview.get_selection()
        model,iter = selection.get_selected()
        
        if iter != None:
            name = model.get_value(iter,0)
        
            if name == "~FOO_fs~":
                self.style_remove.set_sensitive(False)
            else:
                self.style_remove.set_sensitive(True)

    def on_set_treeview_cursor_changed(self,treeview,data=None):
        pass
    
    def on_con_treeview_cursor_changed(self,treeview,data=None):
        pass

    ### add button callbacks
    def on_ops_add_clicked(self,button,data=None):
        name = dialogs.NameDialog(self.window)
        if name:
            if not name in self.TE.get_drawops_list():
                self.add_element(name,"draw_ops")
                return
            else:
                while name in self.TE.get_drawops_list():
                    name += "1"
            
            self.add_element(name,"draw_ops")
            return
            
    def on_geo_add_clicked(self,button,data=None):
        name = dialogs.NameDialog(self.window)
        if name:
            if not name in self.TE.get_geometry_list():
                self.add_element(name,"frame_geometry")
                return
            else:
                while name in self.TE.get_geometry_list():
                    name += "1"
            
            self.add_element(name,"frame_geometry")
            return

    def on_style_add_clicked(self,button,data=None):
        name = dialogs.NameDialog(self.window)
        if name:
            if not name in self.TE.get_frame_style_list():
                self.add_element(name,"frame_style")
                return
            else:
                while name in self.TE.get_frame_style_list():
                    name += "1"
            
            self.add_element(name,"frame_style")
            return

    def on_set_add_clicked(self,button,data=None):
        name = dialogs.NameDialog(self.window)
        if name:
            if not name in self.TE.get_frame_style_set_list():
                self.add_element(name,"frame_style_set")
                return
            else:
                while name in self.TE.get_frame_style_set_list():
                    name += "1"
            
            self.add_element(name,"frame_style_set")
            return
    
    def on_con_add_clicked(self,button,data=None):
        name,value = dialogs.ConstantDialog(self.window)
        if name:
            name = name.capitalize()
            try:
                value = int(value)
            except:
                dialogs.ErrorDialog(_("Error"),
                        _("Value must be an integer"),self.window)
                return

            value = str(value)

            if not name in self.TE.get_constant_list():
                self.add_element(name,"constant",value)
                return
            else:
                while name in self.TE.get_constant_list():
                    name += "1"
            
            self.add_element(name,"constant",value)
            return

    ### remove button callbacks
    def on_ops_remove_clicked(self,button,data=None):
        selection = self.ops_treeview.get_selection()
        model,iter = selection.get_selected()
        
        if iter != None:
            name = model.get_value(iter,0)
            refs = self.TE.remove_element(name,'draw_ops')
            for ref in refs:
                error_msg = (ref[0].upper()+" '"+ref[1]+_("' references ")+name)
                    
                dialogs.ErrorDialog(_("Can not delete"),error_msg,self.window)
            if refs:
                return
            
            model.remove(iter)

    def on_geo_remove_clicked(self,button,data=None):
        selection = self.geo_treeview.get_selection()
        model,iter = selection.get_selected()
        
        if iter != None:
            name = model.get_value(iter,0)
            refs = self.TE.remove_element(name,'frame_geometry')
            for ref in refs:
                error_msg = (ref[0].upper()+" '"+ref[1]+"' references "+name)
                    
                dialogs.ErrorDialog(_("Can not delete"),error_msg,self.window)
            if refs:
                return
            
            model.remove(iter)

        
    def on_style_remove_clicked(self,button,data=None):
        selection = self.style_treeview.get_selection()
        model,iter = selection.get_selected()
        
        if iter != None:
            name = model.get_value(iter,0)
            refs = self.TE.remove_element(name,'frame_style')
            for ref in refs:
                error_msg = (ref[0].upper()+" '"+ref[1]+"' references "+name)
                    
                dialogs.ErrorDialog(_("Can not delete"),error_msg,self.window)
            if refs:
                return
            
            model.remove(iter)

    def on_set_remove_clicked(self,button,data=None):
        selection = self.set_treeview.get_selection()
        model,iter = selection.get_selected()
        
        if iter != None:
            name = model.get_value(iter,0)
            refs = self.TE.remove_element(name,'frame_style_set')
            for ref in refs:
                error_msg = (ref[0].upper()+" '"+ref[1]+"' references "+name)
                    
                dialogs.ErrorDialog(_("Can not delete"),error_msg,
                        self.window)
            if refs:
                return
            
            model.remove(iter)

    def on_con_remove_clicked(self,button,data=None):
        
        selection = self.con_treeview.get_selection()
        model,iter = selection.get_selected()
        
        if iter != None:
            name = model.get_value(iter,0)
            refs = self.TE.remove_element(name,'constant')
            for ref in refs:
                error_msg = (ref[0].upper()+" '"+ref[1]+"' "+
                    _("references ")+name)
                    
                dialogs.ErrorDialog(_("Can not delete"),error_msg,
                        self.window)
            if refs:
                return
            
            model.remove(iter)

#####################
########################

    def reload_ops_liststore(self):
        self.TE.load()
        model = self.ops_treeview.get_model()
        model.clear()

        for name in self.TE.get_drawops_list():
            model.append([name])


    def on_drag_begin(self,treeview,context):
        model = treeview.get_model()
        
        self.order = [row[0] for row in model]

    def on_drag_end(self,treeview,context):
        
        model = treeview.get_model()
        new_order = [row[0] for row in model]

        if new_order == self.order:
            return
    
        child_list = [child for child in self.TE.root]

        ## check to see if ~FOO~ has been moved
        if model[0][0] != "~FOO_do~":
            self.reload_ops_liststore()
            return

        deps = self.TE.reorder_drawops(new_order)
        for dep in deps:
            warning = "Draw ops '"+dep[0]+"' includes '"+dep[1]+"'"
            reorder_anyway = dialogs.ConfirmDialog(_("Warning"),
                    warning,self.window)

            if not reorder_anyway:
                self.reload_ops_liststore()
                return
        
            else:
                self.TE.reorder_drawops(new_order,break_include=True)
        

        u = undo.ChildrenUndo(self.TE.root,child_list)
        
        self.undo_list.append(u)

        self.redo_list = []
        self.ops_undo.set_sensitive(True)
        self.ops_redo.set_sensitive(False)
        
        self.refresh()
    
    ###
    # Undo and redo callbacks
    ###

    def on_ops_undo_clicked(self,button,data=None):
        if not self.undo_list:
            return

        u = self.undo_list.pop()
        u.undo()
        
        self.redo_list.append(u)

        self.reload_ops_liststore()
        
        self.refresh()

        if not self.undo_list:
            self.ops_undo.set_sensitive(False)

        self.ops_redo.set_sensitive(True)

    def on_ops_redo_clicked(self,button,data=None):
        if not self.redo_list:
            return
        
        u = self.redo_list.pop()
        u.redo()

        self.undo_list.append(u)

        self.reload_ops_liststore()

        self.refresh()

        if not self.redo_list:
            self.ops_redo.set_sensitive(False)
        
        self.ops_undo.set_sensitive(True)
