#!/usr/bin/env python

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Authors: Isaiah Heyer (freshapplepy@gmail.com)

# Copyright (C) 2009 Isaiah Heyer

from metacity_themer import GLADE_PATH, RESOURCE_PATH, ICON_PATH
from metacity_themer import CONF_DIR, CT_FILENAME

import pygtk

pygtk.require('2.0')

import gtk
import metacity
import gobject
import gconf
from xml.etree import ElementTree as ET
import os
import sys
import commands
import traceback
import shutil
### my modules

import TE2
import edit
import boxes
import dialogs
import undo
import elements
import utils


class MainWindow(object):
    """
    Main application class
    """
    
    FOO_IMAGE_FILENAME = 'FOO.png'

    def __init__(self):
        
        def check_for_paste():
            if utils.get_can_paste():
                if self.has_theme:
                    self.toolbar_paste_button.set_sensitive(True)
                    return False
            else:
                return True

        gobject.timeout_add(1000,check_for_paste)
   
        self.builder = gtk.Builder()    
        
        try:
            xml_file = utils.join(GLADE_PATH,"gui.xml")
            self.builder.add_from_file(xml_file)
        except Exception,data:
            print _("Failed to load: "),xml_file

        
        try:
            xml_file = utils.join(GLADE_PATH,"example.xml")
            self.builder.add_from_file(xml_file)
        except Exception,data:
            print _("Failed to load: "),xml_file
        
        
        def open_url(dialog,link,data=None):
            
            # get the users preferred browser
            client = gconf.Client()
            cmd = client.get_string('/desktop/gnome/applications/browser/exec')
            output = commands.getoutput(cmd+" "+link)
            utils.log(output)

        gtk.about_dialog_set_url_hook(open_url)

        try:
            xml_file = utils.join(GLADE_PATH,"about_dialog.xml")
            self.builder.add_from_file(xml_file)
        except Exception,data:
            print _("Failed to load: "),xml_file

        

        self.has_theme = False #tell if a theme is loaded

        self.home_dir = os.getenv('HOME')

        #get widgets
        get = self.builder.get_object
        
        self.window = get("window")
        
        ### set window icon
        icon_name = 'metacity-themer'
        icon_theme = gtk.icon_theme_get_default()


        ### check to see if the current theme has a natural 16x16 icon
        iconInfo = icon_theme.lookup_icon(icon_name,16,
                gtk.ICON_LOOKUP_NO_SVG)
        
        if iconInfo:
            # icon is found
            base_size = iconInfo.get_base_size()
            if base_size != 16:
                ## No 16x16 icon so use the default
                icon_theme = gtk.IconTheme()
                icon_theme.set_search_path((ICON_PATH,))
        else:
            icon_theme.append_search_path(ICON_PATH)

        pixbuf = icon_theme.load_icon(icon_name,16,
                gtk.ICON_LOOKUP_NO_SVG)

        gtk.window_set_default_icon(pixbuf)
        ##
        ##
        ##


        self.window.connect("focus-in-event",self.on_window_focus_in)
        self.window.connect("key-press-event",self.on_window_key_press_event)

        self.vbox = get("vbox")
        self.hbox1 = get("hbox1")
        self.vbox1 = get("vbox1")
        self.treeview_xml = get("treeview")
        self.preview_port = get("preview_port")
        self.window_elem_area = get("window_elem_area")
        
        ### enable rgba support
        #screen = self.window.get_screen()
        #colormap = screen.get_rgba_colormap()
        #if colormap:
        #    self.window.set_colormap(colormap)

        self.toolbar_edit_button = get("toolbar_edit")
        self.toolbar_save_button = get("toolbar_save")
        self.toolbar_undo_button = get("toolbar_undo")
        self.toolbar_redo_button = get("toolbar_redo")
        self.toolbar_copy_button = get("toolbar_copy")
        self.toolbar_paste_button = get("toolbar_paste")
        self.toolbar_elements_button = get("toolbar_elements")
        self.toolbar_properties_button = get("toolbar_properties")
        
        self.statusbar = get("statusbar") 
        
        self.window_cbb = get("window_cbb")
        self.element_cbb = get("element_combobox")

        self.prop_viewport = get("prop_viewport")    
        ####

        self.example_box = get("example_box")
       
        ### About Dialog 
        self.dialog = self.builder.get_object("about_dialog")
        ###
        
        #####

        self.window_types = ['normal','dialog','modal_dialog',
                            'menu','utility','border']

        self.types = ['arc','clip','gradient','gtk_arrow','gtk_box',
                'gtk_vline','icon','image','include','line','rectangle',
                'tile','tint','title']
        
        
        
        ##########
        # Build the treeview
        ##########
        self.win_type = None
        self.focus = None
        self.state = None
        self.geometry = None
        self.function = None
        self.Bstate = None

        self.button_order = ['close',
                'maximize',
                'minimize',
                'menu',
                'left_left_background',
                'left_middle_background',
                'left_right_background',
                'right_left_background',
                'right_middle_background',
                'right_right_background']
        
        self.piece_order = ['entire_background',
                'titlebar',
                'titlebar_middle',
                'left_titlebar_edge',
                'right_titlebar_edge',
                'top_titlebar_edge',
                'bottom_titlebar_edge',
                'title',
                'left_edge',
                'right_edge',
                'bottom_edge',
                'overlay']

        self.treestore = gtk.TreeStore(str,str,str,str,str,str)

        win_states = ['normal','maximized','shaded','maximized_and_shaded']
        for state in win_states:
            for focus in ['yes','no']:
                
                if focus == "yes":
                    fstring = "Focused"
                else:
                    fstring = "Unfocused"

                p2 = self.treestore.append(None,[state.upper()+" "+fstring,
                    focus,state,"","",""])
            
                p3b = self.treestore.append(p2,["buttons",focus,
                    state,"Yes","",""])

                p3a = self.treestore.append(p2,["frame pieces",focus,
                    state,"Yes","",""])


                for piece in self.piece_order:
                    p4a = self.treestore.append(p3a,[piece,
                        focus,state,"",piece,""])
                        
                        
                for button in self.button_order:
                    for Bstate in ['normal','prelight','pressed']:
                        p4b = self.treestore.append(p3b,[button+" "+Bstate,
                            focus,state,"",button,Bstate])

        
        #connect signals
        self.builder.connect_signals(self)
        
        self.window.show_all()
                
        ### check to see if the program crashed last time is was run
        crashed_filename = ""

        if os.path.isfile(CT_FILENAME):
            file = open(CT_FILENAME,'r')
            lines = file.readlines()
            i = 0
            for line in lines:
                line = line.strip()
                if line.isdigit():
                    pid = line
                    if not utils.is_pid_running(pid):
                        crashed_filename = lines[i+1].strip()
                        self.theme_name = lines[i+2].strip()

            if not os.path.isfile(crashed_filename):
                crashed_filename = ""
                
                i += 1

            file.close()

        #########
        ## restore the crashed theme if there is one
        if crashed_filename:
            msg = ("Metacity Themer"+
                    _(" crashed while editing theme")+" '"+
                    self.theme_name+"'," +_(" recover?"))

            response = dialogs.ConfirmDialog(_("Theme Recovery"),
                        msg,self.window,True)
            
            if response:
                print _("RECOVERING")
                utils.log("RECOVERING")
                self.load_theme(crashed_filename,restore=True,
                    restore_pid=int(pid))

            else:
                ### Don't ask to recover theme next time
                self.update_ct(int(pid))

    
    def on_add_undo(self):

        self.toolbar_undo_button.set_sensitive(True)

        self.toolbar_redo_button.set_sensitive(False)

    def on_undo_redo(self,undoObject):
        element = undoObject.element

        if element.tag in ['button','piece']:
                
            self.sel_elem = self.TE.draw_ops[element.attrib['draw_ops']]
            self.DrawopsBox.set_element(self.sel_elem)
        
        if self.DrawopsBox.element != None:
            self.DrawopsBox.update()

        if self.FrameBox.cbb:
            self.FrameBox.cbb.update()
            
        if self.FrameGeometry:
            self.FrameGeometry.update()
            
        if self.ButtonGeometry:
            self.ButtonGeometry.update()
            
        if self.WBox:
            self.WBox.update()
            
        if self.FssBox:
            self.FssBox.update()

        if self.focus and self.state:
            # set the preview state
            self.TE.set_preview_state(self.win_type,self.focus,
                self.state)

        else:
            # set the preview state
            self.TE.set_preview_state(self.win_type,"yes","normal")
        ##
        if self.focus and self.state and self.function and self.Bstate:
            self.TE.set_preview_button_state(self.win_type,self.focus,
                self.state,self.function,self.Bstate)


        self.refresh()

        if undo.has_undo():
            self.toolbar_undo_button.set_sensitive(True)
        else:
            self.toolbar_undo_button.set_sensitive(False)
        
        if undo.has_redo():
            self.toolbar_redo_button.set_sensitive(True)
        else:
            self.toolbar_redo_button.set_sensitive(False)
    
    ### misc functions:

    def refresh(self):

        self.TE.write()
        try:
            self.theme_to_preview = metacity.theme_load(self.theme_name)
            self.window_preview.set_theme(self.theme_to_preview)
        except:
            dialogs.ErrorDialog(_("Error"),traceback.format_exc(),self.window)
        else:
            self.TE.write_backup()

        
    def filechooser(self,startpath=None):
        dialog = gtk.FileChooserDialog(_("Open..."),
            self.window,
            gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        
        dialog.set_default_response(gtk.RESPONSE_OK)
        
        last_selected = [False]     ## variable to open last selected
                                    ## valid theme in case the user 
                                    ## starts typing while the dialog
                                    ## is open

        def on_update_preview(file_chooser,preview,last_selected):
            filename = file_chooser.get_preview_filename()
                         
            if not filename:
                file_chooser.set_preview_widget_active(False)
                return

            file_chooser.set_preview_widget_active(True)

            preview_theme_name = filename.split('/')[-1]
            # make sure the theme is valid

            valid, output = utils.theme_validate(preview_theme_name)

            utils.log("theme_test.py output=\n'"+output+"'")

            if valid:
                last_selected[0] = filename
                utils.log("output == Valid")

                theme_preview = metacity.Preview()
                theme_preview.add(gtk.Viewport())
                theme = metacity.theme_load(preview_theme_name)
                theme_preview.set_theme(theme)
                theme_preview.set_title(preview_theme_name)

                for child in preview.get_children():
                    preview.remove(child)

                preview.add(theme_preview)

                theme_preview.realize() # not sure if this is necessary but
                                        # leave it anyway

                preview.show_all()
                
                file_chooser.set_preview_widget_active(True)

                file_chooser.set_response_sensitive(gtk.RESPONSE_OK,True)

            else:
                utils.log("Theme not loaded")

                file_chooser.set_response_sensitive(gtk.RESPONSE_OK,False)

                file_chooser.set_preview_widget_active(False)

                return None
            
            

        preview = gtk.Viewport()
        preview.set_size_request(200,200)
        dialog.set_preview_widget(preview)
        dialog.set_preview_widget_active(False)
        dialog.connect("update-preview",on_update_preview,preview,
                last_selected)

        if startpath != None:
            dialog.set_current_folder(startpath)
        response = dialog.run()
        
        if response == gtk.RESPONSE_OK:
            selected = dialog.get_filename()
            if selected == startpath:
                selected = last_selected[0]

        else:
            selected = None
        
        dialog.destroy()
        return selected
    
    
    
    def ThemeProperties(self):
    
        def on_delete_event(window,data=None):
            for child in elem:
                child.text = old_values[child.tag]

            window.hide_on_delete()
            return True
        
        def on_close_button_clicked(button,elem,old_values):
            for child in elem:
                child.text = old_values[child.tag]

            window.hide()

        def on_focus_in(widget,event,start):
            if type(widget) == gtk.Entry:
                start[0] = widget.get_text()

        def on_focus_out(widget,event,data=None):
            window = widget.get_toplevel()
            if window.has_toplevel_focus():
                widget.activate()

        def on_activate(widget,element,start):
            text = widget.get_text()
            text = text.strip()

            if text == "" and not widget.is_focus(): #check to see if 
                                                #the text is blank
                widget.set_text(start[0]) #and set it to its previous state
                return
            
            element.text = text
            

        def on_save_button_clicked(button,elem,data=None):

            for child in elem:
                text = child.text
                if child.tag == "name":
                
                    parts = self.theme_file.split("/")
                    parts[-3] = text

                    old_dir = utils.get_nparent_dir(self.TE.get_dir(),1)

                    new_dir = os.path.abspath(utils.join(*parts[:-2]))

                    os.rename(old_dir,new_dir)

                    self.theme_file = utils.join(*parts)
                    self.TE.set_theme_file(self.theme_file)

                    if self.index_file:
                        index_filepath = new_dir+self.index_file 

                        lines = open(index_filepath,'r').readlines()

                        newlines = []
                        for l in lines:
                            if len(l.split("=")) == 2:
                                lhs,rhs = l.split("=")
                                rhs = rhs.strip()
                                if (lhs in ['GtkTheme','MetacityTheme']
                                    or lhs.startswith("Name")):

                                    if rhs == self.theme_name:
                                        newlines.append(lhs+"="+text+"\n")
                                    else:
                                        newlines.append(l)
                                else:
                                    newlines.append(l)
                            else:
                                newlines.append(l)
                            
                        file = open(index_filepath,'w')
                            
                        file.write("".join(newlines))

                    self.theme_name = text
                    self.window_preview.set_title(self.theme_name)

            
            self.save_theme()
            
            self.refresh()
            window.hide()


        
        def on_show(window,elem,entries):
            for child in elem:
                entries[child.tag].set_text(child.text)


        ################################## 
        elem = self.TE.get_info_element()

        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        window.set_transient_for(self.window)
        window.set_default_size(300,400)
        window.set_title(_("Edit Theme Info"))

        window.connect("delete_event",on_delete_event)
        entries = {}
        window.connect("show",on_show,elem,entries)

        main_box = gtk.VBox()
        main_box.set_border_width(10)
        
        window.add(main_box)
        
        label = gtk.Label("<b><span size='x-large'>"+"Theme Properties"+
                "</span></b>")

        label.set_use_markup(True)
        
        main_box.pack_start(label,False,False)
        
        old_values = {}

        info_order = ['name','author','copyright','date','description']
        for i in info_order:
            for child in elem:
                if child.tag == i:
                    old_values[i] = child.text

                    box = gtk.HBox()
                    
                    label = gtk.Label()
                    label.set_text("<b>"+i+":</b>")
                    label.set_use_markup(True)
                    label.set_alignment(0.90,0.50)
                    label.set_size_request(100,-1)
                    
                    box.pack_start(label,True,True)

                    entry = gtk.Entry()
                    entries[child.tag] = entry
                    entry.set_text(child.text)
                    start = [child.text]

                    entry.connect("focus-in-event",on_focus_in,start)
                    entry.connect("focus-out-event",on_focus_out)
                    entry.connect("activate",on_activate,child,start)
                    
                    box.pack_start(entry,True,True)

                    main_box.pack_start(box,False,False,4)

        button_box = gtk.HButtonBox()
        button_box.set_layout(gtk.BUTTONBOX_END)

        close_button = gtk.Button(stock=gtk.STOCK_CLOSE)
        save_button = gtk.Button(stock=gtk.STOCK_SAVE)

        close_button.connect("clicked",on_close_button_clicked,elem,old_values)
        save_button.connect("clicked",on_save_button_clicked,elem)
        
        button_box.pack_start(close_button,False,False)
        button_box.pack_start(save_button,False,False)

        button_box.set_child_secondary(close_button,True)

        main_box.pack_end(button_box,False,False)
        
        return window


    def quit(self):

        if self.has_theme:
            if self.TE.get_needs_save():
                reply = dialogs.SaveDialog(self.theme_name,self.window)

                if reply == gtk.RESPONSE_OK:
                    print _("SAVING")
                    utils.log("SAVING")
                    self.save_theme()
            
                elif reply == gtk.RESPONSE_CANCEL:
                    return True


        if os.path.isfile(CT_FILENAME):
            file = open(CT_FILENAME,'r')
            lines = file.readlines()
            write_lines = []
            i = 0
            for line in lines:
                line = line.strip()

                if line.isdigit():
                    pid = int(line)
                    if pid != os.getpid():
                        write_lines.append(line+"\n")
                        write_lines.append(lines[i+1])
                        write_lines.append(lines[i+2])

                i += 1

            file.close()
            
            file = open(CT_FILENAME,'w')
            file.write("".join(write_lines))
            file.close()


        if self.has_theme:
            self.TE.quit()
        
        utils.close_log()

        gtk.main_quit()
        sys.exit(0)

############################################################################

    #define callbacks

############################################################################
    def on_window_delete_event(self,widget,data=None):

        return self.quit()
        
    def on_window_focus_in(self,window,event):
        if not self.has_theme:
            return
        
        # update element comboboxes
        while self.draw_ops_list:
            self.draw_ops_list.pop()

        for name in self.TE.get_drawops_list():
            self.draw_ops_list.append(name)

        self.draw_ops_list.sort()
        
        active = self.element_cbb.get_active()
        model = gtk.ListStore(str)
        for name in self.draw_ops_list:
            model.append([name])

        self.element_cbb.set_model(model)
        self.element_cbb.set_active(active)
        
        try:
            if self.FrameBox.cbb:
                self.FrameBox.cbb.update()
            
            if self.DrawopsBox.element:
                self.DrawopsBox.update()
        except:
            pass


    ## menubar callbacks
    
    def on_menu_open_activate(self,widget,data=None):
        self.open()

    def on_menu_save_activate(self,widget,data=None):
        if self.has_theme:
            self.save_theme()

    def on_menu_quit_activate(self,widget,data=None):

        self.quit()

    def on_about_dialog_delete_event(self,dialog,data=None):
        dialog.hide_on_delete()

    def on_menu_about_activate(self,widget,data=None):
        
        self.dialog.set_transient_for(self.window)
        self.dialog.set_resizable(True)

        response = self.dialog.run()
        
        if response == gtk.RESPONSE_CANCEL:
            self.dialog.hide()

        return

    ###########################
    ## keyboard shortcuts
    ###########################

    def on_window_key_press_event(self,window,event):
        ## undo and redo
        if event.keyval == 122:
            if event.state == gtk.gdk.CONTROL_MASK | gtk.gdk.MOD2_MASK:
                undo.undo()
        
        elif event.keyval == 121:
            if event.state == gtk.gdk.CONTROL_MASK | gtk.gdk.MOD2_MASK:
                undo.redo()
        #####

        ## save
        elif event.keyval == 115:
            if event.state == gtk.gdk.CONTROL_MASK | gtk.gdk.MOD2_MASK:
                if self.has_theme:
                    self.save_theme()

        
    def on_treeview_cursor_changed(self,widget,data=None):
            
        def on_cbb_changed(widget,element):
        
            old_attrib = element.attrib.copy()

            if element.tag == "button":
                at = widget.get_active_text()
                self.TE.set_button_drawops(self.win_type,self.focus,
                        self.state,self.function,self.Bstate,at)

                self.TE.set_preview_button_state(self.win_type,self.focus,
                        self.state,self.function,self.Bstate)
            
            elif element.tag == "piece":
                element.attrib['draw_ops'] = widget.get_active_text()
            
            self.sel_elem = self.TE.draw_ops[widget.get_active_text()]

            self.DrawopsBox.set_element(self.sel_elem)
            self.FrameBox.label.set_text(label_text)
            self.FrameBox.label.set_use_markup(True)

            if element.attrib != old_attrib:
                u = undo.ButtonDrawopsUndo(self.TE,self.win_type,self.focus,
                    self.state,self.function,self.Bstate,old_attrib)

                undo.add_undo(u)

            self.refresh()
            

        self.element_cbb.set_active(-1)
        self.selected = self.treeview_xml.get_selection()
        Iter = self.selected.get_selected()[1]

        if type(Iter) != gtk.TreeIter:
            return
        model = self.treeview_xml.get_model()
        
        self.value = model.get_value(Iter,0)

        self.win_type = self.window_types[self.window_cbb.get_active()]

        self.focus = model.get_value(Iter,1)
        self.state = model.get_value(Iter,2)
        self.geometry = model.get_value(Iter,3)
        self.function = model.get_value(Iter,4)
        self.Bstate = model.get_value(Iter,5)

        elem = self.TE.Get_Element(self.win_type,self.focus,self.state,
                self.geometry,self.function,self.Bstate)
        
        if self.focus and self.state:
            # set the preview state
            self.TE.set_preview_state(self.win_type,self.focus,
                    self.state)

        else:
            # set the preview state
            self.TE.set_preview_state(self.win_type,"yes","normal")

        self.elem = elem
            
        a = elem.attrib
        
        if self.preview_port.get_child() == self.DrawopsBox:
            if elem is self.old_elem and self.value == self.old_value:
                self.refresh()
                return

                
        self.old_elem = elem
        self.old_value = self.value
        

            
        ###################################################################
        ## if elem is a frame element
        ###################################################################
        
        if elem.tag == "frame":
            
            self.FssBox = edit.Frame(self.refresh,self.win_type,elem,
                    self.TE)
                
            ### remove old box
            child = self.prop_viewport.get_child()
            if child:
                self.prop_viewport.remove(child)
                    
            self.prop_viewport.add(self.FssBox)
            
            self.FssBox.show()

            self.refresh()
            return
        ###################################################################
        ###################################################################
        ## if elem is a frame_geometry element
        ###################################################################
        ###################################################################

        elif elem.tag == "frame_geometry":
                
            
            if self.value == "buttons":
                
                geo_box = edit.ButtonGeometry(self.refresh,elem)
                self.ButtonGeometry = geo_box

                ### remove old box
                child = self.prop_viewport.get_child()
                if child:
                    self.prop_viewport.remove(child)

                    
                if geo_box.get_parent():
                    geo_box.get_parent().remove(geo_box)

                self.prop_viewport.add(geo_box)
                geo_box.show_all()
                    
                self.refresh()
                return

            elif self.value == "frame pieces":
                    
                geo_box = edit.FrameGeometry(self.refresh,elem)
                self.FrameGeometry = geo_box


                ### remove old box
                child = self.prop_viewport.get_child()
                if child:
                    self.prop_viewport.remove(child)

                    
                if geo_box.get_parent():
                    geo_box.get_parent().remove(geo_box)

                self.prop_viewport.add(geo_box)
                geo_box.show_all()
                    
                self.refresh()
                return

        ###################################################################
        ###################################################################
        ## if elem is a frame style child element
        ###################################################################
        ###################################################################

        ##frame style parts:
        elif elem.tag == "piece":
            label_text = ("<span size='large'><b>"+a['position']+
                "</b></span>")

                    
        elif elem.tag == "button":
            # change the preview
            self.TE.set_preview_button_state(self.win_type,self.focus,
                    self.state,self.function,self.Bstate)
                
            if self.value.find("restore") == -1:
                label_text = ("<span size='large'><b>"+a['function']+
                    " "+a['state']+"</b></span>")
            else:
                label_text = ("<span size='large'><b>"+'restore'+
                    " "+a['state']+"</b></span>")
            
        else:

            ### remove old box
            child = self.prop_viewport.get_child()
            if child:
                self.prop_viewport.remove(child)

            self.refresh()
            return
            
        self.sel_elem = self.TE.draw_ops[elem.attrib['draw_ops']]
        
        if not self.prop_viewport.get_child() is self.FrameBox:
            ### remove old box
            child = self.prop_viewport.get_child()
            if child:
                self.prop_viewport.remove(child)
            
            if not self.DrawopsBox in self.FrameBox.get_children():
                self.FrameBox.pack_start(self.DrawopsBox,True,True)

            self.prop_viewport.add(self.FrameBox)

        self.DrawopsBox.set_element(self.sel_elem)
        self.FrameBox.label.set_text(label_text)
        self.FrameBox.label.set_use_markup(True)

        if self.FrameBox.cbb:
            self.FrameBox.cbb.set_element(elem)
        else:
            cbb = boxes.AttribComboBox(self.refresh,
                elem,"draw_ops",self.draw_ops_list,
                    on_cbb_changed)

            self.FrameBox.cbb_box.add(cbb)
            self.FrameBox.cbb = cbb


        self.refresh()
        self.prop_viewport.show_all()


    def on_element_combobox_changed(self,widget,data=None):
        
        if widget.get_active() == -1:
            return

        name = widget.get_active_text()

        self.sel_elem = self.TE.draw_ops[name]

        if not self.prop_viewport.get_child() is self.DrawopsBox:
            ### remove old box
            child = self.prop_viewport.get_child()
            if child:
                self.prop_viewport.remove(child)
            
            if self.DrawopsBox.get_parent():
                self.FrameBox.remove(self.DrawopsBox)
            self.prop_viewport.add(self.DrawopsBox)
        
        self.DrawopsBox.set_element(self.sel_elem)
        self.prop_viewport.show_all()
    
    def on_window_cbb_changed(self,cbb,data=None):

        self.win_type = self.window_types[cbb.get_active()]
        
        elem = self.TE.Get_Element(self.win_type)

        self.WBox = edit.Window(self.refresh,elem,self.TE)
                
        ### remove old box
        child = self.window_elem_area.get_child()
        if child:
            self.window_elem_area.remove(child)
                    
        self.window_elem_area.add(self.WBox)
            
        self.WBox.show()

        if self.focus and self.state:
            # set the preview state
            self.TE.set_preview_state(self.win_type,self.focus,self.state)
        else:
            self.TE.set_preview_state(self.win_type,"yes","normal")

        self.refresh()

    def on_toolbar_open_clicked(self,button,data=None):
        self.open()
    
    def open(self):
        if self.has_theme:
            if self.TE.get_needs_save():
                reply = dialogs.SaveDialog(self.theme_name,self.window)

                if reply == gtk.RESPONSE_OK:
                    self.save_theme()
            
                elif reply == gtk.RESPONSE_CANCEL:
                    return

        
        theme_dir = utils.join(self.home_dir,".themes")

        dirpath = self.filechooser(theme_dir)
        if dirpath == None:
            return None

        self.load_theme(dirpath)
            

    def on_toolbar_save_clicked(self,button,data=None):

        self.save_theme()

    def on_toolbar_redo_clicked(self,widget,data=None):
        widget.set_flags(gtk.CAN_FOCUS)
        widget.grab_focus()
        undo.redo()

    def on_toolbar_undo_clicked(self,widget,data=None):
        widget.set_flags(gtk.CAN_FOCUS)
        widget.grab_focus()
        undo.undo()
    

    def on_toolbar_copy_clicked(self,widget,data=None):

        ### only copy if a drawops is currently being edited
        child = self.prop_viewport.get_child()
        if not child:
            return
        for c in child.get_children():
            if type(c) == edit.Drawops:
                self.DrawopsBox.copy()
                self.toolbar_paste_button.set_sensitive(True)

    def on_toolbar_paste_clicked(self,widget,data=None):

        ### only paste if a drawops is currently being edited
        child = self.prop_viewport.get_child()
        for c in child.get_children():
            if type(c) == edit.Drawops:
                self.DrawopsBox.paste()

    def on_toolbar_elements_clicked(self,button,data=None):
        ew = elements.Editor(self.TE,self.window,self.refresh)
    
    def on_toolbar_properties_clicked(self,button,data=None):
        self.TP.show_all()

    
    def update_ct(self,restore_pid):
        """
        Remove crash data no longer relevent 
        from file CT_FILENAME
        """

        if os.path.isfile(CT_FILENAME):
            file = open(CT_FILENAME,'r')
            lines = file.readlines()
            write_lines = []
            i = 0
            for line in lines:
                line = line.strip()
                if line.isdigit():
                    pid = int(line)

                    if pid != os.getpid():
                        if pid != restore_pid:
                            if utils.is_pid_running(pid):
                                write_lines.append(line+"\n")
                                write_lines.append(lines[i+1])
                                write_lines.append(lines[i+2])

                i += 1

            file.close()
            
            file = open(CT_FILENAME,'w')
            file.write("".join(write_lines))
            file.close()



    def load_theme(self,theme_dir,restore=False,restore_pid=None):
        
        # make sure to not to load a theme thats already open
        # in another instance of this program

        self.theme_name = theme_dir.split('/')[-1]

        dirpath = utils.join(self.home_dir,".themes",self.theme_name)

        theme_file = utils.join(theme_dir,'metacity-1',
                'metacity-theme-1.xml')

        Iter = os.walk(theme_dir)
        for i in Iter:
            for file in i[2]:
                if file == "index.theme":
                    self.index_file = file

        
        self.update_ct(restore_pid)

        if not restore:
            if os.path.isfile(CT_FILENAME):
                for line in open(CT_FILENAME,'r'):
                    line = line.strip()
                    if line == theme_file:
                        response = dialogs.ConfirmDialog(_("Warning"),
                            _("Theme is already open in another window,\n")+
                            _("open anyway?"),self.window)

                        if not response:
                            return None
          
        ### undo stuff ###
        undo.clear()

        undo.set_add_undo_callback(self.on_add_undo)
        undo.set_undo_callback(self.on_undo_redo)
        undo.set_redo_callback(self.on_undo_redo)


        if self.has_theme:
            self.TE.quit()
        
        self.theme_file = theme_file
        self.TE = TE2.Theme(theme_file,restore)

        ###create a file to list the last opened theme
        # in case of crash
        ###
        
        
        crashed_theme_info = open(CT_FILENAME,'a')

        crashed_theme_info.write(str(os.getpid())+"\n")
        crashed_theme_info.write(theme_file+"\n")   
        crashed_theme_info.write(self.theme_name+"\n")   

        crashed_theme_info.close()

        self.ButtonGeometry = None
        self.FrameGeometry = None
        
        self.WBox = None
        self.FssBox = None

        self.old_elem = None
        self.old_value = None

        # make the toolbar buttons sensitive to input
        self.toolbar_save_button.set_sensitive(True)
        
        self.toolbar_undo_button.set_sensitive(False)
        self.toolbar_redo_button.set_sensitive(False)

        self.toolbar_copy_button.set_sensitive(True)
        self.toolbar_paste_button.set_sensitive(False)
        
        self.toolbar_elements_button.set_sensitive(True)
        self.toolbar_properties_button.set_sensitive(True)
        
        
        ## default image file for 'image' draw operations
        foo_path = utils.join(RESOURCE_PATH,'images',
                self.FOO_IMAGE_FILENAME)
        
        if not os.path.isfile(foo_path):
            utils.log("Error FOO.png image not found")

        theme_foo_path = utils.join(self.TE.get_dir(),
                self.FOO_IMAGE_FILENAME)

        if not os.path.isfile(theme_foo_path):
            shutil.copy(foo_path,theme_foo_path)
        
        self.has_theme = True

        self.focus_state = "focused"
        
        ### set treemodels
        
        self.draw_ops_list = self.TE.get_drawops_list()
        self.draw_ops_list.sort()

        model = gtk.ListStore(str)
        for name in self.draw_ops_list:
            model.append([name])

        self.element_cbb.set_model(model)
       
        
        # the window preview
        self.window_preview = metacity.Preview()
        
        self.example_box.reparent(self.window_preview)
        
        # remove old widgets if any
        for child in self.prop_viewport.get_children():
            self.prop_viewport.remove(child)

        for child in self.preview_port.get_children():
            self.preview_port.remove(child)


    
        self.preview_port.add(self.window_preview)
        
        self.window_preview.realize()
        
        
        self.theme_to_preview = metacity.theme_load(self.theme_name)

        self.window_preview.set_theme(self.theme_to_preview)
        self.window_preview.set_title(self.theme_name)
        
        ### 
        self.window_cbb.set_sensitive(True)
        self.window_cbb.set_active(0)
        
        self.element_cbb.set_sensitive(True)
        ####

        ### Build the treeview
        for column in self.treeview_xml.get_columns():
            self.treeview_xml.remove_column(column)

        self.cell_xml = gtk.CellRendererText()
        
        self.tvcolumn1 = gtk.TreeViewColumn("",self.cell_xml)
        self.tvcolumn2 = gtk.TreeViewColumn("",self.cell_xml)

        self.tvcolumn2.set_visible(False)
        
        self.treeview_xml.set_headers_visible(False)
        ## make the rows alternately colored
        self.treeview_xml.set_rules_hint(True)
        
        self.treeview_xml.append_column(self.tvcolumn1)
        self.treeview_xml.append_column(self.tvcolumn2)
        
        self.treeview_xml.set_model(self.treestore)
        self.tvcolumn1.add_attribute(self.cell_xml,"text",0)
        self.tvcolumn2.add_attribute(self.cell_xml,"text",1)
        
        #show everything
        self.window.show_all()

        
        ################################################### 
        self.win_type = self.window_types[self.window_cbb.get_active()]
        
        elem = self.TE.Get_Element(self.win_type)

        self.WBox = edit.Window(self.refresh,elem,self.TE)
                
        ### remove old box
        child = self.window_elem_area.get_child()
        if child:
            self.window_elem_area.remove(child)
                    
        self.window_elem_area.add(self.WBox)
            
        self.WBox.show()

        if self.focus and self.state:
            # set the preview state
            self.TE.set_preview_state(self.win_type,self.focus,self.state)
        else:
            self.TE.set_preview_state(self.win_type,"yes","normal")

        self.refresh()

        ################################################################


        self.TP = self.ThemeProperties()
        self.DrawopsBox = edit.Drawops(self.refresh,
                self.TE)

        
                
        self.FrameBox = gtk.VBox()

        label = gtk.Label("<b>!!!</b>")
        label.set_use_markup(True)
        self.FrameBox.pack_start(label,False,False)
        self.FrameBox.label = label

        cbb_box = gtk.HBox()
        self.FrameBox.cbb_box = cbb_box
        self.FrameBox.cbb = None
        
        self.FrameBox.pack_start(cbb_box,False,False)
        self.FrameBox.pack_start(self.DrawopsBox,True,True)

    def save_theme(self):
        if self.has_theme:
            self.TE.save()


def main(theme2open=None):
    
    mt = MainWindow()

    if theme2open:
        theme2open = utils.join(os.getenv('HOME'),'.themes',theme2open)
        mt.load_theme(theme2open)

    gtk.main()

if __name__ == "__main__":
    run()
