#!/usr/bin/env python

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Authors: Isaiah Heyer (freshapplepy@gmail.com)

# Copyright (C) 2008 Isaiah Heyer

import re

def _color_check(colorstring):
    
    c = colorstring
    c.strip()
    if c[0] == "#":
        c = c[1:]
    
    if len(c) == 6 or len(c) == 3:
        try:
            int(c,16)
        except ValueError:
            return False
        else:
            return True
    else:
        return False
        
def is_valid(expr,consts):
    
    if len(expr) < 1:
        return (False,"empty expression")
    original = expr
    has_mm = False


    if expr[0] == "-":
        return (False,"Negative number")
    
    #look for html color codes:
    re_color = re.compile('#[\w]+')
    matchIter = re_color.finditer(expr)
    for color in matchIter:
        if not _color_check(expr[color.start():color.end()]):
            msg = "invalid color '"+expr[color.start():color.end()]+"'"
            return (False,msg)
    expr = re_color.sub('2',expr)
    
    for c in consts:
        c = re.escape(c)
        pattern = re.compile(c)
        expr = pattern.sub("1 ",expr)
    
    """
    for m in ['`max`','`min`']:
        
        l = expr.split(m)
        if len(l) < 2:
            break
        left_side = l[0]
        
        pl = 0
        
        left_side_has_const = False       
        for i in xrange(len(left_side)-1,-1,-1):

            if left_side[i] == "(":

                if pl == 0:
                    left_side = left_side[i+1:]
                    break
                pl -= 1

            elif left_side[i] == ")":
                pl += 1 
                
        for c in consts:
            if left_side.find(c) > -1:
                left_side_has_const = True        
                break

        if not left_side_has_const:
            try:
                result = eval(left_side)
                if result < 0:
                    return (False,"Negative value")
            except Exception,data:
                print data
        ###################
        ###################
        for right_side in l[1:]:

            right_side_has_const = False       
            pr = 0

            for i in xrange(len(right_side)-1):
                if right_side[i] == ")":
                    if pr == 0:
                        right_side = right_side[:i]
                        break
                    pr -= 1

                elif right_side[i] == "(":
                    pr += 1 
            
            for c in consts:
                if right_side.find(c) > -1:
                    right_side_has_const = True        
                    break

            if not right_side_has_const:
                try:
                    result = eval(right_side)
                    if result < 0:
                        return (False,"Negative value")
                except Exception,data:
                    print "right "+str(data)


"""
    
    for c in ['`max`','`min`']:
        c = re.escape(c)
        pattern = re.compile(c)
        old = expr
        expr = pattern.sub("*",expr)
        if expr != old:
            has_mm = True

    for c in ['blend','shade']:
        c = re.escape(c)
        pattern = re.compile(c)
        old = expr
        expr = pattern.sub("2",expr)
        if expr != old:
            has_mm = True

    for c in ['object_width','object_height']:
        if c not in consts:
            string = re.escape(c)
            pattern = re.compile(string)
            if re.findall(pattern,expr):
                msg = "No constant named '"+c+"'"
                return (False,msg)
        
        
    has_const = False
    if original != expr:
        has_const = True
    
    try:
        result = eval(expr)
        if result < 0 and not has_const:
            return (False,"Negative value")
        #elif result < 0 and has_mm and not has_const: 
        #    return (False,"Negative value")
        
    except SyntaxError,data:
        print expr
        return (False,"invalid syntax")

    except NameError,data:
        msg = "No constant named " + data.message.split()[1] 
        return (False,msg)

    except Exception,data:
        return (False,data)
    else:
        return (True,None)

    
if __name__ == "__main__":
    import time
    start = time.time()
    print is_valid("((3 `max` (width-title_width)) / 2)",['gtk:text[ACTIVE]','height'])
    print time.time() - start
