#!/usr/bin/env python
import os
import commands
import sys
from metacity_themer import THEME_TEST_PATH, LOG_FILENAME, DB_FILENAME, PID
import pickle
import anydbm

def theme_validate(theme_name):

    theme_test_path = join(THEME_TEST_PATH,'theme_test.py')

    output = commands.getoutput("python '"+theme_test_path+"' '"+
            theme_name+"'")

    output = output.strip()

    if output == "Valid":
        if theme_name in os.listdir(join(os.getenv('HOME'),'.themes')):
            return (True,output)

    return (False,output)
            
def join(path, *paths):
    """
    short hand function for os.path.join that also
    runs os.path.normpath on the path
    """
    if path == "":
        path = '/'

    p = os.path.join(path,*paths)
    p = os.path.normpath(p)

    return p

def get_nparent_dir(path,n):
    """
    Returns the nth parent directory of path 
    """
    
    sep = os.path.sep

    parts = path.split(sep)

    n = len(parts) - n

    path = sep.join(parts[:n])
    return path

def is_pid_running(pid):
    """
    Returns True if pid (Process ID) is still running
    """

    pid = str(pid)
    output = commands.getoutput('ps -A')
    for line in output.split('\n'):
        if pid in line and PID in line:
            return True
    else:
        return False


def copy_elems(obj_list):
    """
    Copies objects in obj_list to an element database
    """
    db_filename = join(DB_FILENAME)
    if os.path.isfile(db_filename):
        os.remove(db_filename)
        
    dbmOut = anydbm.open(db_filename,'n')

    for obj in obj_list:
        k = pickle.dumps(obj,2)
        dbmOut[k] = k
    
    dbmOut.close()

def get_can_paste():

    try:
        db = anydbm.open(DB_FILENAME)
    except Exception,data:
        return False
    else:
        db.close()
        return True

def paste_elems():
    """
    Returns elems in the element database
    """
    db_filename = join(DB_FILENAME)

    dbmIn = anydbm.open(db_filename,'r')
    
    ret_list = []
    for value in dbmIn.values():
        elem = pickle.loads(value)
        ret_list.append(elem)
        
    dbmIn.close()

    return ret_list


logfile = open(LOG_FILENAME,'a+')

def log(string):
    """
    Log arg string to metacity_themer.LOG_FILENAME
    """

    logfile.write("\n"+("=" * 80)+"\n")
    logfile.write(string)

def close_log():
    logfile.close()

